#!D:\bld\xfab_1644315860030\_h_env\python.exe

"""
 Runs makemap.py multiple times, removes grains without enough peaks
 Should enter nine variables: nX.ubi, n(X-1).flt, nX.flt, initial cuts, final cuts,
 	stop limit, min number of peaks, full path to .par file, crystal symmetry
"""

from __future__ import absolute_import
from __future__ import print_function
import sys
import os
from string import split,digits,letters#,replace
from six.moves import range


# read and check input file
if len(sys.argv) != 10:
	print("Should enter nine variables:") 
	print("nX.ubi, n(X-1).flt, nX.flt, ")
	print("initial cuts, final cuts, stop limit, ")
	print("min number of peaks, full path to .par file, crystal symmetry \n") 
	print("Example:")
	print("makemap_all.py nall.ubi n00.flt rest.flt 0.03 0.02 5 100 NF_final.par cubic\n")
	sys.exit()
else:
	UBI = sys.argv[1]			# Input grains in .ubi format, overwritten by the program
	FLT_IN = sys.argv[2]		# Input peaks in .flt format
	FLT_OUT = sys.argv[3]		# Unassignted peaks in .flt format
	ICUTS=eval(sys.argv[4])		# Initial cuts, deviation from integer hkl, typically 0.02
	FCUTS=eval(sys.argv[5])		# Final cuts, deviation from integer hkl, typically 0.01
	LIMIT=eval(sys.argv[6])     # Minimum difference in remaining peaks after running makemap.py
	MINPEAKS=eval(sys.argv[7])  # Minimum number of peaks a seed grain needs to be retained in the final list
	PARS=sys.argv[8]            # Full path to the parameters file
	CRYSYMM=sys.argv[9]         # Crystal symmetry, e.g. "cubic"


# Run makemap loop 1 with first threshold cut ICUTS
print("Beginnning makemap loop 1 with cuts (-t %f)" %ICUTS)
command = "makemap.py -p %s -u %s -U %s -f %s -F %s -t %f -s %s > makemapTempOut.txt" %(PARS,UBI,UBI,FLT_IN,FLT_OUT,ICUTS,CRYSYMM)
os.system(command)

f = open("makemapTempOut.txt",'r')
lines = f.readlines()
f.close()

pks0 = eval(lines[-2].translate(None,letters))
pks1 = eval(lines[-1].translate(None,letters))
print("Initial number of peaks %i" %pks0)
print("Number of remaining peaks %i" %pks1)
	
new_assigned = LIMIT + 1
counter = 1
while new_assigned > LIMIT:	
	counter = counter + 1
	command = "makemap.py -p %s -u %s -U %s -f %s -F %s -t %f -s %s > makemapTempOut.txt" %(PARS,UBI,UBI,FLT_IN,FLT_OUT,ICUTS,CRYSYMM)
	os.system(command)
	f = open("makemapTempOut.txt",'r')
	lines = f.readlines()
	f.close()
	pks2 = eval(lines[-1].translate(None,letters))
	new_assigned = pks1 - pks2
	pks1 = pks2

print("makemap.py was run %i times" %(counter))


# Remove grains with less than MINPEAKS peaks
f = open(UBI,'r')
lines = f.readlines()
f.close()

breaklinenum = -1
for i in range(len(lines)):
	if "npks" in lines[i]:
		npks = eval(lines[i][1:].translate(None,letters))
		if npks < MINPEAKS:
			breaklinenum = i - 3
			break
if breaklinenum > 0:
	print("Removing grains with less than %i peaks" %MINPEAKS)
	print("Deleting from line %i on!" %breaklinenum)
	f = open(UBI,'w')
	for i in range(breaklinenum):
		f.write("%s" %lines[i])
	f.close()
elif breaklinenum == 0:
	print("All seed grains are bad.")
	print("Deleting %s and %s files!" %(UBI,FLT_OUT))
	os.remove(UBI)
	os.remove(FLT_OUT)
else:
	print("All grains have at least %i peaks" %MINPEAKS)		
			
# Run makemap loop 2 with first threshold cut FCUTS
if os.path.exists(UBI):
	print("Beginnning makemap loop 2 with cuts (-t %f)" %FCUTS)
	command = "makemap.py -p %s -u %s -U %s -f %s -F %s -t %f -s %s > makemapTempOut.txt" %(PARS,UBI,UBI,FLT_IN,FLT_OUT,FCUTS,CRYSYMM)
	os.system(command)

	f = open("makemapTempOut.txt",'r')
	lines = f.readlines()
	f.close()

	pks0 = eval(lines[-2].translate(None,letters))
	pks1 = eval(lines[-1].translate(None,letters))
	print("Initial number of peaks %i" %pks0)
	print("Number of remaining peaks %i" %pks1)
	
	new_assigned = LIMIT + 1
	counter = 1
	while new_assigned > LIMIT:	
		counter = counter + 1
		command = "makemap.py -p %s -u %s -U %s -f %s -F %s -t %f -s %s > makemapTempOut.txt" %(PARS,UBI,UBI,FLT_IN,FLT_OUT,FCUTS,CRYSYMM)
		os.system(command)
		f = open("makemapTempOut.txt",'r')
		lines = f.readlines()
		f.close()
		pks2 = eval(lines[-1].translate(None,letters))
		new_assigned = pks1 - pks2
		pks1 = pks2

	print("makemap.py was run %i more times" %(counter))

os.remove("makemapTempOut.txt")


"""
xfab.symmetry has a few function for calculation of symmetry
related stuff
"""

from __future__ import absolute_import
from __future__ import print_function
from xfab import tools
from six.moves import range
import numpy as np
from xfab import checks
from xfab import CHECKS
from xfab import xfab_logging
logger = xfab_logging.get_module_level_logger(__name__)


def Umis(umat_1, umat_2, crystal_system):
    """Compute the misorientation between two rotation matrices.

    `umat_2` is seen in light of all the possible indistinguishable symmetries generated by the
    input `crystal_system`. For each symmetry a new rotation matrix can be created and compared
    to `umat_1`. The misorientations are computed as

        arccos( trace(umat_2 . umat_1.T) - 1)/2 )

    Args:
        umat_1 (:obj:numpy array): Rotation matrix to be compared to `umat_2`, ``shape=(3,3)``
        umat_2 (:obj:numpy array): Rotation matrix to be compared to `umat_2`, ``shape=(3,3)``
        crystal_system (:obj:int): crystal_system number must be one of 1: Triclinic, 2: Monoclinic,
            3: Orthorhombic, 4: Tetragonal, 5: Trigonal, 6: Hexagonal, 7: Cubic

    Returns:
        (:obj:numpy array): misorientation between `umat_1` and `umat_2` for each
            symmetry operation allowed by the input crystal system in units of degrees
            ``shape=(N,2)``.

    """
    if CHECKS.activated:
        checks._check_rotation_matrix(umat_1)
        checks._check_rotation_matrix(umat_2)

    rot = ROTATIONS[crystal_system]
    misorientations = np.empty((len(rot), 2))

    # The below line of code is the faster equivalent of
    # lengths = [ 0.5 * trace( dot( r, dot( u1.T  , u2 ) ) - 0.5 for r in rot ]
    # Note that the trace is invariant under cyclic permutations.
    # Note that applying an element of rot upon an input orientation matrix
    # produces a new orientation matrix that, given the input crystal_system,
    # will appear to be identical to the starting orientation.
    # see xfab.symmetry.rotations() for further details.
    lengths = 0.5 * (rot * np.dot(umat_1.T, umat_2)).sum(axis=(1, 2)) - 0.5

    misorientations[:, 0] = np.arange(len(rot))
    misorientations[:, 1] = np.arccos(lengths.clip(-1, 1)) * 180./np.pi
    return misorientations


def add_perm(hkl, crystal_system):
    """
    apply the permutation according to the crystal system on the
    hkl and print the result
    """

    perm = permutations(crystal_system)
    nperm = perm.shape[0]

    for k in range(nperm):
        logger.debug(np.dot(perm[k],hkl))


def add_rot(umat, crystal_system):
    """
    apply the rotation according to the crystal system on the
    orientation matrix U and print the result
    """

    rot = rotations(crystal_system)
    nrot = rot.shape[0]

    for k in range(nrot):
        logger.debug(np.dot(umat, rot[k]))


def permutations(crystal_system):

    """
    permutations returns the set of indistinguasible lattice permutations

    perm = permutations(crystal_system)
    hkl_new = perm*hkl

    crystal_system can be one of the following values
    1: Triclinic
    2: Monoclinic
    3: Orthorhombic
    4: Tetragonal
    5: Trigonal
    6: Hexagonal
    7: Cubic

    Henning Osholm Sorensen, Riso, 30/6/2006
    Implemented in python, 12/7/2008
    """

    if crystal_system < 1 or crystal_system > 7:
        raise ValueError('Crystal system shoud have a value between 1 and 7')

    if crystal_system == 1: # Triclinic
        perm = np.zeros((1, 3, 3))
        perm[0]  = [[ 1., 0, 0], [ 0, 1., 0], [ 0, 0, 1.]]

    if crystal_system == 2: # Monoclinic
        perm = np.zeros((2, 3, 3))
        perm[0]  = [[ 1, 0, 0], [ 0, 1, 0], [ 0, 0,  1]]
        perm[1]  = [[-1, 0, 0], [ 0, 1, 0], [ 0, 0, -1]]

    if crystal_system == 3: # Orthorhombic
        perm = np.zeros((4, 3, 3))
        perm[0]  = [[ 1, 0, 0], [ 0,  1, 0], [ 0, 0,  1]]
        perm[1]  = [[-1, 0, 0], [ 0, -1, 0], [ 0, 0,  1]]
        perm[2]  = [[-1, 0, 0], [ 0,  1, 0], [ 0, 0, -1]]
        perm[3]  = [[ 1, 0, 0], [ 0, -1, 0], [ 0, 0, -1]]

    if crystal_system == 4: # Tetragonal
        perm = np.zeros((8, 3, 3))
        perm[0]  = [[ 1,  0, 0], [ 0,  1, 0], [ 0, 0,  1]]
        perm[1]  = [[-1,  0, 0], [ 0, -1, 0], [ 0, 0,  1]]
        perm[2]  = [[ 0,  1, 0], [-1,  0, 0], [ 0, 0,  1]]
        perm[3]  = [[ 0, -1, 0], [ 1,  0, 0], [ 0, 0,  1]]
        perm[4]  = [[-1,  0, 0], [ 0,  1, 0], [ 0, 0, -1]]
        perm[5]  = [[ 1,  0, 0], [ 0, -1, 0], [ 0, 0, -1]]
        perm[6]  = [[ 0,  1, 0], [ 1,  0, 0], [ 0, 0, -1]]
        perm[7]  = [[ 0, -1, 0], [-1,  0, 0], [ 0, 0, -1]]

    if crystal_system == 5: # Trigonal
        perm = np.zeros((6, 3, 3))
        perm[0]  = [[ 1,  0, 0], [ 0,  1, 0], [ 0, 0,  1]]
        perm[1]  = [[ 0,  1, 0], [-1, -1, 0], [ 0, 0,  1]]
        perm[2]  = [[-1, -1, 0], [ 1,  0, 0], [ 0, 0,  1]]
        perm[3]  = [[ 0,  1, 0], [ 1,  0, 0], [ 0, 0, -1]]
        perm[4]  = [[ 1,  0, 0], [-1, -1, 0], [ 0, 0, -1]]
        perm[5]  = [[-1, -1, 0], [ 0,  1, 0], [ 0, 0, -1]]

    if crystal_system == 6: # Hexagonal
        perm = np.zeros((12, 3, 3))
        perm[0]  = [[ 1,  0, 0], [ 0,  1, 0], [ 0, 0,  1]]
        perm[1]  = [[ 0,  1, 0], [-1, -1, 0], [ 0, 0,  1]]
        perm[2]  = [[-1, -1, 0], [ 1,  0, 0], [ 0, 0,  1]]
        perm[3]  = [[-1,  0, 0], [ 0, -1, 0], [ 0, 0,  1]]
        perm[4]  = [[ 0, -1, 0], [ 1,  1, 0], [ 0, 0,  1]]
        perm[5]  = [[ 1,  1, 0], [-1,  0, 0], [ 0, 0,  1]]
        perm[6]  = [[ 0,  1, 0], [ 1,  0, 0], [ 0, 0, -1]]
        perm[7]  = [[ 1,  0, 0], [-1, -1, 0], [ 0, 0, -1]]
        perm[8]  = [[-1, -1, 0], [ 0,  1, 0], [ 0, 0, -1]]
        perm[9]  = [[ 0, -1, 0], [-1,  0, 0], [ 0, 0, -1]]
        perm[10] = [[-1,  0, 0], [ 1,  1, 0], [ 0, 0, -1]]
        perm[11] = [[ 1,  1, 0], [ 0, -1, 0], [ 0, 0, -1]]

    if crystal_system == 7: # Cubic
        perm = np.zeros((24, 3, 3))
        perm[0]  = [[ 1,  0,  0], [ 0,  1,  0], [ 0,  0,  1]]
        perm[1]  = [[ 1,  0,  0], [ 0, -1,  0], [ 0,  0, -1]]
        perm[2]  = [[ 1,  0,  0], [ 0,  0, -1], [ 0,  1,  0]]
        perm[3]  = [[ 1,  0,  0], [ 0,  0,  1], [ 0, -1,  0]]
        perm[4]  = [[-1,  0,  0], [ 0,  1,  0], [ 0,  0, -1]]
        perm[5]  = [[-1,  0,  0], [ 0, -1,  0], [ 0,  0,  1]]
        perm[6]  = [[-1,  0,  0], [ 0,  0, -1], [ 0, -1,  0]]
        perm[7]  = [[-1,  0,  0], [ 0,  0,  1], [ 0,  1,  0]]
        perm[8]  = [[ 0,  1,  0], [-1,  0,  0], [ 0,  0,  1]]
        perm[9]  = [[ 0,  1,  0], [ 0,  0, -1], [-1,  0,  0]]
        perm[10] = [[ 0,  1,  0], [ 1,  0,  0], [ 0,  0, -1]]
        perm[11] = [[ 0,  1,  0], [ 0,  0,  1], [ 1,  0,  0]]
        perm[12] = [[ 0, -1,  0], [ 1,  0,  0], [ 0,  0,  1]]
        perm[13] = [[ 0, -1,  0], [ 0,  0, -1], [ 1,  0,  0]]
        perm[14] = [[ 0, -1,  0], [-1,  0,  0], [ 0,  0, -1]]
        perm[15] = [[ 0, -1,  0], [ 0,  0,  1], [-1,  0,  0]]
        perm[16] = [[ 0,  0,  1], [ 0,  1,  0], [-1,  0,  0]]
        perm[17] = [[ 0,  0,  1], [ 1,  0,  0], [ 0,  1,  0]]
        perm[18] = [[ 0,  0,  1], [ 0, -1,  0], [ 1,  0,  0]]
        perm[19] = [[ 0,  0,  1], [-1,  0,  0], [ 0, -1,  0]]
        perm[20] = [[ 0,  0, -1], [ 0,  1,  0], [ 1,  0,  0]]
        perm[21] = [[ 0,  0, -1], [-1,  0,  0], [ 0,  1,  0]]
        perm[22] = [[ 0,  0, -1], [ 0, -1,  0], [-1,  0,  0]]
        perm[23] = [[ 0,  0, -1], [ 1,  0,  0], [ 0, -1,  0]]

    return perm


def rotations(crystal_system):

    """
    rotations returns the set of unitary rotation matrices
    corresponding to the indistinguasible lattice permutations
    The values of the function only differ from permutations for
    trigonal and hexagonal crystal systems

    rot = rotations(crystal_system)
    U_new = U*rot

    U_new*B*perm*hkl = U*B*hkl, so U_new = U*B*perminv*Binv and
    rot = B*perminv*Binv. If B or perm is diagonal, rot = perminv.
    perminv included in perm, but to get the i'th entry of rot
    to correspond to the i'th entry of perm one must use perminv.

    crystal_system can be one of the following values
    1: Triclinic
    2: Monoclinic
    3: Orthorhombic
    4: Tetragonal
    5: Trigonal
    6: Hexagonal
    7: Cubic

    Jette Oddershede, Riso, 8/2/2010
    """

    if crystal_system < 1 or crystal_system > 7:
        raise ValueError('Crystal system shoud have a value between 1 and 7')

    if crystal_system == 1: # Triclinic
        rot = permutations(crystal_system)
        for i in range(len(rot)):
            rot[i] = rot[i].T

    if crystal_system == 2: # Monoclinic
        rot = permutations(crystal_system)
        for i in range(len(rot)):
            rot[i] = rot[i].T

    if crystal_system == 3: # Orthorhombic
        rot = permutations(crystal_system)
        for i in range(len(rot)):
            rot[i] = rot[i].T

    if crystal_system == 4: # Tetragonal
        rot = permutations(crystal_system)
        for i in range(len(rot)):
            rot[i] = rot[i].T

    if crystal_system == 5: # Trigonal
        perm = permutations(crystal_system)
        B = tools.form_b_mat([1.,1.,1.,90.,90.,120.])
        Binv = np.linalg.inv(B)
        rot = np.zeros((6, 3, 3))
        for i in range(len(perm)):
            rot[i] = np.dot(B,np.dot(np.linalg.inv(perm[i]),Binv))

    if crystal_system == 6: # Hexagonal
        perm = permutations(crystal_system)
        B = tools.form_b_mat([1.,1.,1.,90.,90.,120.])
        Binv = np.linalg.inv(B)
        rot = np.zeros((12, 3, 3))
        for i in range(len(perm)):
            rot[i] = np.dot(B,np.dot(np.linalg.inv(perm[i]),Binv))

    if crystal_system == 7: # Cubic
        rot = permutations(crystal_system)
        for i in range(len(rot)):
            rot[i] = rot[i].T

    return rot

ROTATIONS = [None] + [np.ascontiguousarray(rotations(i)) for i in range(1,8)]




"""
The xfab.detector modules provides a numbe of function 
for calculation of detector coordinates, detector coordinate 
transformations etc.

"""

from __future__ import absolute_import
import numpy as n

def det_coor(Gt, costth, wavelength, distance, y_size, z_size, 
             dety_center, detz_center, R_tilt, tx, ty, tz):
    """
    Calculates detector coordinates dety,detz

    INPUT:
    Gt is the g-vector 
    y_size and z-size are the detector pixel size in y, and z (in microns)
    (dety_center, detz-center) is the beam center of the detector (in pixels)
    R_tilt is the rotation matrix of the detector
    (tx, ty, tz) is the position of the grain at the present omega

    OUTPUT:
    [dety, detz]
    """

    # Unit directional vector for reflection
    v = n.array([costth, 
                 wavelength/(2*n.pi)*Gt[1],
                 wavelength/(2*n.pi)*Gt[2]])
    t = (R_tilt[0, 0]*distance - \
             n.sum(R_tilt[:, 0]*n.array([tx, ty, tz])))/n.sum(R_tilt[:, 0]*v)
    Ltv = n.array([tx-distance, ty, tz])+ t*v
    dety = n.sum(R_tilt[:, 1]*Ltv)/y_size + dety_center
    detz = n.sum(R_tilt[:, 2]*Ltv)/z_size + detz_center
    return [dety, detz]

def det_coor2(tth, eta, distance, y_size, z_size, 
              dety_center, detz_center, R_tilt, tx, ty, tz,):
    """
    Calculates detector coordinates dety,detz
    Alternative to det_coor using the angles the azimuthal angle eta
    for calculation of the detector coordinates instead of the g-vector

    INPUT:
    tth is two-theta
    eta is the azimuthal angle around the beam center ccw with eta= 0 at +ylab
    y_size and z-size are the detector pixel size in y, and z (in microns)
    (dety_center, detz-center) is the beam center of the detector (in pixels)
    R_tilt is the rotation matrix of the detector
    (tx, ty, tz) is the position of the grain at the present omega

    OUTPUT:
    [dety, detz]
    """
    costth = n.cos(tth)
    sintth = n.sin(tth)
    # Unit directional vector for reflection
    v = n.array([costth, 
                 -sintth*n.sin(eta),
                 sintth*n.cos(eta)])
    t = (R_tilt[0, 0]*distance - \
             n.sum(R_tilt[:, 0]*n.array([tx, ty, tz])))/n.sum(R_tilt[:, 0]*v)
    Ltv = n.array([tx-distance, ty, tz])+ t*v
    dety = n.sum(R_tilt[:, 1]*Ltv)/y_size + dety_center
    detz = n.sum(R_tilt[:, 2]*Ltv)/z_size + detz_center
    return [dety, detz]

def det_v(Gt, costth, wavelength, distance, y_size, z_size, 
             dety_center, detz_center, R_tilt, tx, ty, tz):
    """
    Calculates direction of outgoing X-ray beam (used by fabric)
    needs to be cleaned up

    INPUT:
    Gt is the g-vector 
    y_size and z-size are the detector pixel size in y, and z (in microns)
    (dety_center, detz-center) is the beam center of the detector (in pixels)
    R_tilt is the rotation matrix of the detector
    (tx, ty, tz) is the position of the grain at the present omega

    OUTPUT:
    [dety, detz]
    """

    # Unit directional vector for reflection
    v = n.array([costth, 
                 wavelength/(2*n.pi)*Gt[1],
                 wavelength/(2*n.pi)*Gt[2]])
    return v


def detector_to_lab(dety, detz, L, py, pz, y0, z0, R_tilt):
    """
    calculates the laboratory coordinates (xl,yl,zl) from the detector
    coordinates (dety,detz), the sample-to-detector distance L, the pixel
    sizes (py and pz), the beam centre (y0,z0) and the detector tilt R_tilt 
    """
       
    lab = n.array([[L], [0], [0]]) + n.dot(R_tilt, n.array([[0],
                                                            [py*(dety-y0)],
                                                            [pz*(detz-z0)]]))
    return [lab[0][0], lab[1][0], lab[2][0]]
    

def trans_orientation(img, o11, o12, o21, o22, flipdir = 'forward'):
    """
    Transforming image matrix according to the 
    detector orientation matrix given the 
    output image  matrix will have coordinates (dety,detz)
    as defined in 
    "3DXRD and TotalCryst Geometry - Version 1.0.2" by
    H.F. Poulsen, S. Schmidt, J. Wright, H.O. Sorensen

    It is important to note that the image is not only flipped 
    using the orientation matrix, but also transposed such the 
    the returned array img has the coordinates [dety,detz].
    Therefore even the identy orientation matrix performs a transformation.
    To avoid this please use the function image_flipping
    
    Detector_orientation: [[o11,o12],[o21,o22]]
    if pretransposed to get img -> img(dety,detz) then
           [[o11,o12],[o21,o22]]
           [[  1,  0],[  0,  1]]  => nothing
           [[ -1,  0],[  0,  1]]  => fliplr
           [[  1,  0],[  0, -1]]  => flipud
           [[ -1,  0],[  0, -1]]  => flipud fliplr
    These will not be pretransposed
           [[  0,  1],[  1,  0]]  => nothing
           [[  0, -1],[ -1,  0]]  => fliplr flipud
           [[  0, -1],[  1,  0]]  => fliplr
           [[  0,  1],[ -1,  0]]  => flipud
           
    flipdir can takes the values forward or inverse
    forward: raw image -> 3DXRD standard
    inverse: 3DXRD standard -> raw image
 
    """

    if abs(o11) == 1:
        if (abs(o22) != 1) or (o12 != 0) or (o21 != 0):
            raise ValueError('detector orientation makes no sense 1')
        img = n.transpose(img) # to get A[i,j] be standard A[dety,detz] 
        if o11 == -1:
            if flipdir == 'forward':
                img = n.fliplr(img)
            else: #inverse direction from (dety,detz) to imageformat
                # since these direction of operations should be obversed
                # we should now do flipud before transpose 
                # But transpose(fliplr(img)) = flipud(transpose(img)) 
                img = n.flipud(img)
        if o22 == -1:
            if flipdir == 'forward':
                img = n.flipud(img)
            else: #inverse direction from (dety,detz) to imageformat
                img = n.fliplr(img)
        return img
    if abs(o12) == 1:
        if abs(o21) != 1 or (o11 != 0) or (o22 != 0):
            raise ValueError('detector orientation makes no sense 2')
        #transpose not needed since the matrix is transp from scratch
        if o12 == -1:
            img = n.fliplr(img)
        if o21 == -1:
            img = n.flipud(img)
        return img
    raise ValueError('detector orientation makes no sense 3')


def image_flipping(img, o11, o12, o21, o22, flipdir='forward'):
    """
    Transforming image matrix according to the 
    detector orientation matrix given the 
    output image  matrix will have coordinates (dety,detz)
    as defined in 
    "3DXRD and TotalCryst Geometry - Version 1.0.2" by
    H.F. Poulsen, S. Schmidt, J. Wright, H.O. Sorensen
    
    Detector_orientation: [[o11,o12],[o21,o22]]

           [[o11,o12],[o21,o22]]
           [[  1,  0],[  0,  1]]  => nothing
           [[ -1,  0],[  0,  1]]  => flipud
           [[  1,  0],[  0, -1]]  => fliplr
           [[ -1,  0],[  0, -1]]  => flipud fliplr

           [[  0,  1],[  1,  0]]  => transpose
           [[  0, -1],[ -1,  0]]  => transpose fliplr flipud
           [[  0, -1],[  1,  0]]  => transpose flipud
           [[  0,  1],[ -1,  0]]  => transpose flipud
           
    flipdir can takes the values forward or inverse
    forward: raw image -> 3DXRD standard
    inverse: 3DXRD standard -> raw image

    """

    if abs(o11) == 1:
        if (abs(o22) != 1) or (o12 != 0) or (o21 != 0):
            raise ValueError('detector orientation makes no sense 1')
#        img = n.transpose(img) # to get A[i,j] be standard A[dety,detz] 
        if o11 == -1:
            img = n.flipud(img)
        if o22 == -1:
            img = n.fliplr(img)
        return img
    if abs(o12) == 1:
        if abs(o21) != 1 or (o11 != 0) or (o22 != 0):
            raise ValueError('detector orientation makes no sense 2')
        #transpose not needed since the matrix is transp from scratch
        img = n.transpose(img) # make transpose
        
        if o12 == -1:
            if flipdir == 'forward':
                img = n.flipud(img)
            else:
                img = n.fliplr(img)
        if o21 == -1:
            if flipdir == 'forward':
                img = n.fliplr(img)
            else:
                img = n.flipud(img)
        return img
    raise ValueError('detector orientation makes no sense 3')



def detyz_to_xy(coor, o11, o12, o21, o22, dety_size, detz_size):
    """
    Transforming dety, detz coordinates to meet (x,y) coordinates
    of the raw image according to the  detector orientation matrix given.
    The definition of (dety,detz) is defined in 
    "3DXRD and TotalCryst Geometry - Version 1.0.2" by
    H.F. Poulsen, S. Schmidt, J. Wright, H.O. Sorensen
    
    Detector_orientation: o= [[o11,o12],
                              [o21,o22]]
    returns (dety, detz)
    """
    

    if abs(o11) == 1:
        if (abs(o22) != 1) or (o12 != 0) or (o21 != 0):
            raise ValueError('detector orientation makes no sense 1')
    elif abs(o12) == 1:
        if abs(o21) != 1 or (o11 != 0) or (o22 != 0):
            raise ValueError('detector orientation makes no sense 2')
    else:
        raise ValueError('detector orientation makes no sense 3')
    # transpose (dety, detz) to (detz, dety) to match order of (x,y)
    coor = n.array([coor[1], coor[0]])
    omat = n.array([[o11, o12], 
                    [o21, o22]])
    # Well we need to use the inverse operations here
    omat = n.linalg.inv(omat)
    det_size = n.array([detz_size-1, 
                        dety_size-1]) # also transpose coord in size
    coor = n.dot(omat, coor) - n.clip(n.dot(omat, det_size), 
                                      -n.max(det_size), 0)
    return coor

def xy_to_detyz(coor, o11, o12, o21, o22, dety_size, detz_size):
    """
    Transforming (x,y) coordinates of the raw image 
    to (dety, detz) coordinates according to the  
    detector orientation matrix given.
    The definition of (dety,detz) is defined in 
    "3DXRD and TotalCryst Geometry - Version 1.0.2" by
    H.F. Poulsen, S. Schmidt, J. Wright, H.O. Sorensen
    
    Detector_orientation: o= [[o11,o12],
                              [o21,o22]]
    returns (dety,detz)

    """
    

    if abs(o11) == 1:
        if (abs(o22) != 1) or (o12 != 0) or (o21 != 0):
            raise ValueError('detector orientation makes no sense 1')
    elif abs(o12) == 1:
        if abs(o21) != 1 or (o11 != 0) or (o22 != 0):
            raise ValueError('detector orientation makes no sense 2')
    else:
        raise ValueError('detector orientation makes no sense 3')
    omat = n.array([[o11, o12],
                    [o21, o22]])
    det_size = n.array([detz_size-1,
                        dety_size-1])
    coor = n.dot(omat, coor)- n.clip(n.dot(omat, det_size),
                                     -n.max(det_size), 0)
    # transpose (x,y) to (y,x) to match order of (dety,detz)
    coor = n.array([coor[1], coor[0]])
    return coor
     
     
def detyz_to_eta_and_radpix(coor, dety_center, detz_center):
    """
    Transforming (dety,detz) coordinates to (eta,radpix),
    where eta is in degrees (clockwise, zero point at 12 o'clock)
    and radpix is the radial number of pixels from the beam center.
    """
    
    radcoor = coor - n.array([dety_center,detz_center])
    radpix = n.sqrt(n.sum(radcoor**2))
    if radpix < 1:
        cos_eta = 1
    else:
        cos_eta = radcoor[1]/radpix
    if radcoor[0] <= 0:
        eta = 180/n.pi*n.arccos(cos_eta)
    else:
        eta = 360-180/n.pi*n.arccos(cos_eta)
        
    return [eta,radpix]
    

def eta_and_radpix_to_detyz(eta, radpix, dety_center, detz_center):
    """
    Transforming (eta,radpix) to detector coordinates (dety.detz)
    where eta is in degrees (clockwise, zero point at 12 o'clock)
    and radpix is the radial number of pixels from the beam center.
    """
    
    etarad = eta*n.pi/180.
    radcoor = radpix*n.array([-n.sin(etarad),n.cos(etarad)])
    coor = radcoor + n.array([dety_center,detz_center])
    
    return coor
    

def distort(coor, o11, o12, o21, o22, dety_size, detz_size, spatial):
    """
    To match the coordinate system of the spline file
    """

    (x, y) = detyz_to_xy(coor,
                         o11,
                         o12,
                         o21,
                         o22,
                         dety_size,
                         detz_size)

    # Do the spatial distortion
    (xd, yd) = spatial.distort(x, y)
    
    # transform coordinates back to dety,detz
    coord = xy_to_detyz([xd, yd],
                        o11,
                        o12,
                        o21,
                        o22,
                        dety_size,
                        detz_size)
    return coord



#!D:\bld\xfab_1644315860030\_h_env\python.exe


'''
tweakdetpars

This script optimizes the spatial detector-sample parameters so that
the number of peaks found is maximized. The peak-finding is applied
to a select number of seed grains (.ubi file), which is specified
as this script is called.

The script should be called like so:

tweakdetpars FF_fit.par m00.flt mall.ubi 0.02 2 FF_final.par -x FF
tweakdetpars NF_fit.par peaks_min10_t100.flt n01.ubi 0.03 3 NF_final.par -x NF

where 0.02 is the hkl tolerance and 2 is the number of iterations.

User input, such as parameter steps and range, can be found below the
function definitions.

'''

from __future__ import absolute_import
from __future__ import print_function
import argparse
import os
import shutil
import sys # where timeStep and layer are stored
import textwrap

import numpy as np
from scipy.optimize import leastsq
import matplotlib
from six.moves import map
from six.moves import range
matplotlib.use('PDF')
import matplotlib.pyplot as plt # for plotting data if needed
import string

from ImageD11.refinegrains import parameters, refinegrains, transform


def scor_old(r): # find number of peaks
    '''
    Finds number of peaks for specified detector-sample parameter values.

    Parameters:
        r = parameter values
    Returns:
        n = number of peaks found
        e = ?
    '''

    #r.assignlabels() ## works here... try on line 202 instead

    npks = 0
    drlv = 0
    ks = list(r.grains.keys())
    for key in ks:
        g = r.grains[key]
        grainname = key[0]
        scanname = key[1]
        g.peaks_xyz = transform.compute_xyz_lab(
            [g.sc, g.fc],
            **r.parameterobj.parameters
            )
        # compute gv using current parameters, including grain position
        r.set_translation( key[0], key[1] )
        r.compute_gv(g)
        r.refine(g.ubi)
        npks += r.npks
        drlv += r.avg_drlv2
    return npks, drlv


def scor_maggie(r):
    # slightly modified version of refinegrains.gof
    drlv = 0.
    npks = 0.

    # defaulting to fitting all grains
    for key in r.grains.keys():

        g = r.grains[key]
        grainname = key[0]
        scanname = key[1]

        # Compute gv using current parameters
        # Keep labels fixed
        g.peaks_xyz = transform.compute_xyz_lab(
            [g.sc, g.fc],
            **r.parameterobj.parameters
            )
        r.compute_gv(g)

        g.set_ubi(r.refine(r.ubisread[grainname]))

        drlv += r.avg_drlv2
        npks += r.npks
    return npks, drlv


def scor_new(r):
    r.grains_to_refine = list(r.grains.keys())
    r.recompute_xlylzl = True
    return r.gof(), 0
    
def scor(r):
    """
    This scor function replaces all of the above since it should not refine grains (ubi and trans), but only assign reflections
    """
    r.assignlabels(quiet=True) 
    npks = np.sum(labels > -1 for labels in r.scandata[args.fltfile].labels)
    drlv = (np.sum(r.scandata[args.fltfile].drlv2)-(len(r.scandata[args.fltfile].drlv2)-npks))/npks
    return npks, drlv


def gaussian(x, *p):
    '''
    Creates y values for a specified Gaussian curve.

    Parameters:
        x = array of x values
        p* = array of Gaussian parameters (A, mu, sigma)
    Returns:
        y = array of y values for Gaussian curve
    '''
    A, mu, sigma = p
    y = A*np.exp(-(x-mu)**2/(2.*sigma**2))
    return y


def residuals(p, y, x): # difference between points and Gaussian curve
    A, mu, sigma = p
    err = y - A*np.exp(-(x-mu)**2/(2.*sigma**2))
    return err


def gauss_fit(x, y):
    '''
    Creates Gaussian fit of xy points.

    Parameters:
        x = array of x values (must be same # of elements as y!)
        y = array of y values (must be same # of elements as x!)
    Returns:
        p_opt = array of Gaussian parameters (A, mu, sigma)

    '''
    if y.std() == 0:
        # can't fit something that isn't varying
        return

    # set initial Gaussian parameters
    mu = x[np.argmax(y)] # set initial mu to x value of maximum
    A = np.amax(y) # set initial amplitude to y value of maximum
    # set initial sigma to x-distance from mu after 34.1% of curve area
    total_area = y.sum() # area under curve
    area_from_center = 0.
    sigma = 0
    for i in range( np.argmax(y), len(y) ):
        area_from_center += y[i]
        if area_from_center > .341 * total_area:
            sigma = x[i] - x[np.argmax(y)]
            break
    if sigma == 0:
        print('Range does not span a stddev, try increasing the range')
        sigma =  x[-1] - x[np.argmax(y)]
    p0 = [A, mu, sigma]
    print('Initial estimates of Gaussian fit parameters:')
    print(('A     %.1f' % A))
    print(('mu    %.4f' % mu))
    print(('sigma %.4f' % sigma))

    # find optimal Gaussian parameters
    #p_opt = curve_fit(gaussian, x, y, p0=p0)
    return leastsq(residuals, p0, args=(y, x))[0]


def main(args):

    # set up peak-finding
    print()
    r = refinegrains()
    r.loadparameters(args.parfile)
    r.loadfiltered(args.fltfile)
    r.readubis(args.ubifile)
    r.generate_grains()
    r.tolerance = float(args.tol)
    p = parameters.parameters()
    p.loadparameters(args.parfile)


    #*********************** BEGIN USER INPUT ***********************#

    print()
    param_range = np.array([
    #     MIN          MAX
    #----------------------------
     [ r.parameterobj.parameters['distance'] * (1-args.dd),
       r.parameterobj.parameters['distance'] * (1+args.dd), ],     #dd
     [ r.parameterobj.parameters['y_center'] - args.dy,
       r.parameterobj.parameters['y_center'] + args.dy,     ],     #dy
     [ r.parameterobj.parameters['z_center'] - args.dz,
       r.parameterobj.parameters['z_center'] + args.dz,     ],     #dz
     [ r.parameterobj.parameters['tilt_x']   - args.dtx,
       r.parameterobj.parameters['tilt_x']   + args.dtx,    ],     #dtx
     [ r.parameterobj.parameters['tilt_y']   - args.dty,
       r.parameterobj.parameters['tilt_y']   + args.dtz,    ],     #dty
     [ r.parameterobj.parameters['tilt_z']   - args.dtz,
       r.parameterobj.parameters['tilt_z']   + args.dtz,    ],     #dtz
     [ r.parameterobj.parameters['wedge']    - args.dw,
       r.parameterobj.parameters['wedge']    + args.dw,     ]  ])  #dw

    #************************* END USER INPUT *************************#


    # read and record initial optimal paramter values
    init_pars = open(args.parfile,'r')
    init_pars_array = np.genfromtxt(init_pars,usecols=1)
    distance = init_pars_array[8]
    y_center = init_pars_array[25]
    z_center = init_pars_array[27]
    tilt_x = init_pars_array[20]
    tilt_y = init_pars_array[21]
    tilt_z = init_pars_array[22]
    wedge = init_pars_array[24]

    p_opt_array = [distance,y_center,z_center,tilt_x,tilt_y,tilt_z,wedge]

    # create list of parameter names

    all_param_names = ['distance', 'y_center', 'z_center',
                       'tilt_x', 'tilt_y', 'tilt_z', 'wedge']
    param_names = []
    if args.dd != 0:
        param_names.append('distance')
    if args.dy != 0:
        param_names.append('y_center')        
    if args.dz != 0:
        param_names.append('z_center')        
    if args.dtx != 0:
        param_names.append('tilt_x')        
    if args.dty != 0:
        param_names.append('tilt_y')        
    if args.dtz != 0:
        param_names.append('tilt_z')        
    if args.dw != 0 and (args.experiment == 'FF' or args.experiment == 'ff'):
        param_names.append('wedge')        


    print('varying:')
    print(param_names)
    print()

    # find number of peaks using initial optimal parameter values
    for i, name in enumerate(all_param_names):
        r.parameterobj.parameters[name] = p_opt_array[i]
    r.assignlabels() ## try here...
    n, e = scor(r)
    print(('Peaks found before parameter refinement: %d' % n))
    print()

    # parameter refinement iteration loop
    for current_iteration in range(args.iterations):

        if current_iteration > 0:
        ## Update range on parameters
            param_range[0,0] = r.parameterobj.parameters['distance']*(1-args.dd)
            param_range[0,1] = r.parameterobj.parameters['distance']*(1+args.dd)
            param_range[1,0] = r.parameterobj.parameters['y_center']-args.dy
            param_range[1,1] = r.parameterobj.parameters['y_center']+args.dy
            param_range[2,0] = r.parameterobj.parameters['z_center']-args.dz
            param_range[2,1] = r.parameterobj.parameters['z_center']+args.dz
            param_range[3,0] = r.parameterobj.parameters['tilt_x']-args.dtx
            param_range[3,1] = r.parameterobj.parameters['tilt_x']+args.dtx
            param_range[4,0] = r.parameterobj.parameters['tilt_y']-args.dty
            param_range[4,1] = r.parameterobj.parameters['tilt_y']+args.dty
            param_range[5,0] = r.parameterobj.parameters['tilt_z']-args.dtz
            param_range[5,1] = r.parameterobj.parameters['tilt_z']+args.dtz
            param_range[6,0] = r.parameterobj.parameters['wedge']-args.dw
            param_range[6,1] = r.parameterobj.parameters['wedge']+args.dw
    # skip parameter refinement?
        if args.skip_par_ref:
            print('Option to skip parameter refinement is set to ON.'
                  'GrainSweeper will use parameters values from the nf_X.par'
                  'file, unless the opt_par_values_nf_X.txt already exists '
                  '(in which case values from this .txt file will be used.')
            print()
            print('The parameter refinement skipping option can be '
                  'toggled/overridden in the tweakpars.py script.')
            print()
            break

            ## set initial optimal parameters
            for i, name in enumerate(all_param_names):
                r.parameterobj.parameters[name] = p_opt_array[i]

        # create 3D array where:
        #     rows = parameter variation increments
        #     cols = parameter value, peaks found, e (3 cols total)
        #     slices = parameters varied
        param_peaks = np.zeros( shape= (args.steps, 3, len(param_names)) )

        ## find peaks for each par variation, fit Gaussian to curve, and find
        ## optimal par values
        for i, name in enumerate(param_names): # for each parameter
            print()
            print(('Varying parameter: %s; value: %g' % (name, p_opt_array[all_param_names.index(name)])))
            print('   distance   y_center  z_center    tilt_x      tilt_y'
                  '      tilt_z      wedge     peaks found     e')
            var_range = list(np.linspace(param_range[all_param_names.index(name),0], param_range[all_param_names.index(name),1],
                                         args.steps))
            for j, d in enumerate(var_range): # each parameter variation step
                r.parameterobj.parameters[name] = d
                dd = r.parameterobj.parameters['distance']
                yy = r.parameterobj.parameters['y_center']
                zz = r.parameterobj.parameters['z_center']
                tx = r.parameterobj.parameters['tilt_x']
                ty = r.parameterobj.parameters['tilt_y']
                tz = r.parameterobj.parameters['tilt_z']
                ww = r.parameterobj.parameters['wedge']

                n, e = scor(r)
                param_peaks[j,:,i] = d, n, e # write values to array
                print((
                    '%11.1f %9.1f %9.1f %11.6f %11.6f %11.6f %11.6f %10d '
                    '%11.5f' % (dd, yy, zz, tx, ty, tz, ww, n, e)
                    ))

            # create Gaussian fit
            p_opt = gauss_fit( param_peaks[:,0,i], param_peaks[:,1,i] )
            if p_opt is None:
                print('cant fit something that doesnt vary')
                r.parameterobj.parameters[name] = p_opt_array[all_param_names.index(name)]
                continue

            # create Gaussian xy values, and plot original and fit curves
            # together
            x_gauss_steps = np.linspace(param_range[all_param_names.index(name),0], param_range[all_param_names.index(name),1],
                                        200 )
            y_gauss_steps = gaussian(x_gauss_steps, *p_opt)
            plt.xlabel(str(name))
            plt.ylabel('peaks found')
            plt.plot(param_peaks[:,0,i], param_peaks[:,1,i])
            plt.plot(x_gauss_steps, y_gauss_steps)
            if args.experiment:
                figurename = ('%s_param_var_%s_iter%d.pdf'
                              % (args.experiment, name,
                                 current_iteration+1)
                              )
            else:
                figurename = ('NEW_param_var_%s_iter%d.pdf'
                              % (name, current_iteration+1)
                              )
                plt.savefig(figurename)
                plt.clf()

            ## print and record newest updated optimal parameter value,
            ## print peaks found
            print(('Optimal parameter value: %.4f' % p_opt[1]))
            p_opt_array[all_param_names.index(name)] = p_opt[1]
            r.parameterobj.parameters[name] = p_opt_array[all_param_names.index(name)]
            print()
            n, e = scor(r) # find number of peaks using all optimal
                           #parameter values
            print(('Peaks found after variation: %d' % n))

        # print optimal parameters from current iteration
        print()
        print(('Optimal parameter values for parameter refinement'
              'iteration %d:' % (current_iteration+1)))
        print((' '.join(map(str, all_param_names))))
        print(('  '.join(map(str, p_opt_array))))
        print()

    # create optimal parameter values file (for importing into shell)
    if args.experiment:
        filename = '%s_opt_par_values.ini' % (args.experiment)
        writeline = args.experiment + '=' + str(n)
    else:
        filename = 'NEW_opt_par_values.ini'
        writeline = 'NEW = ' + str(n)

    text_file = open(filename, 'w')
    for i, name in enumerate(all_param_names):
        text_file.write( str(name) + '=' + str(p_opt_array[i]) + '\n')

    text_file.write(writeline)
    text_file.close()

    # write new paramterfile
    r.parameterobj.parameters['t_x'] = 0
    r.parameterobj.parameters['t_y'] = 0
    r.parameterobj.parameters['t_z'] = 0
    r.parameterobj.saveparameters(args.outparfile)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        prog='tweakdetpars',
        description='Tweaks detector parameters to optimize reflections per '
                     'grain.',
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog=textwrap.dedent('''
            examples:

            $ tweakdetpars FF_fit.par m00.flt mall.ubi 0.02 2 FF_final.par -x FF
            $ tweakdetpars NF_fit.par n00.flt nall.ubi 0.03 3 NF_final.par -x NF
            ''')
        )
    parser.add_argument(
        'parfile', type=str, help='name and path to starting detector par file'
        )
    parser.add_argument(
        'fltfile', type=str, help='name and path to flt file'
        )
    parser.add_argument(
        'ubifile', type=str, help='name and path to ubi file'
        )
    parser.add_argument(
        'tol', type=float, help='hkl tolerance'
        )
    parser.add_argument(
        'iterations', type=int, help='the number of iterations'
        )
    parser.add_argument(
        'outparfile', type=str, help='name and path to output detector par file'
        )
    parser.add_argument(
        '-s', '--steps', type=int, default=25,
        help='number of steps in each optimization calculation'
        )
    parser.add_argument(
        '-x', '--experiment', type=str, choices=['nf', 'ff', 'NF', 'FF'],
        default='', help='near-field or far-field experiment'
        )
    parser.add_argument(
        '-v', '--verbose', action='store_true',
        help='report progress in terminal'
        )
    parser.add_argument(
        '-k', '--skip-par-ref', action='store_true',
        help='skip the refinement process, uses the initial optimal par values'
        )
    parser.add_argument(
        '-dd', '--dd', type=float, default=0.05,
        help='range for tweaking parameter distance'
        )
    parser.add_argument(
        '-dy', '--dy', type=float, default=10,
        help='range for tweaking parameter y-center'
        )
    parser.add_argument(
        '-dz', '--dz', type=float, default=10,
        help='range for tweaking parameter z-center'
        )
    parser.add_argument(
        '-dtx', '--dtx', type=float, default=0.05,
        help='range for tweaking parameter tilt_x'
        )
    parser.add_argument(
        '-dty', '--dty', type=float, default=0.1,
        help='range for tweaking parameter tilt_y'
        )
    parser.add_argument(
        '-dtz', '--dtz', type=float, default=0.1,
        help='range for tweaking parameter tilt_z'
        )
    parser.add_argument(
        '-dw', '--dw', type=float, default=1,
        help='range for tweaking parameter wedge (not refined for NF)'
        )
    args = parser.parse_args()
    main(args)



#!D:\bld\xfab_1644315860030\_h_env\python.exe

from __future__ import absolute_import
from __future__ import print_function
import argparse
import os
import shutil
import sys
import textwrap

from ImageD11 import columnfile as ic
from six.moves import range


def process_layer(args, layer):
    ## Checks for existance of directory for saving files
    pdir = '%s/%02d' % (args.output, layer)
    if os.path.exists(pdir):
        if args.force:
            shutil.rmtree(pdir)
        else:
            raise IOError('%s already exists' % pdir)
    ## Creats directory for saving files
    os.mkdir(pdir)

    ## Defines first image and last image number in layer
    first_im =  args.nstart + args.nimages * layer###
    last_im = args.nimages + first_im -1

    ## Creats median file using fable 'median.py'
    ndigits = None
    for i in range(7):
        fmtstr = '%s%0' + str(i) + 'd%s'
        first_im_name = fmtstr % (args.stem, args.nstart, args.image_format)
        if os.path.exists(first_im_name):
            ndigits = i
            break
    if ndigits is None:
        raise RuntimeError('format string problem, revisit')
    ## defines directory path for raw data/images
    ## first image, last image, step size between images
    if args.image_format == '.ge2':
        medianskip = int(args.nimages/240)
        args.medianstep = 1
    else:
        medianskip = int(args.nimages/args.medianstep)
    command = ('median.py -i %s -f %i -l %i -s %i -o %s/median' %
               (first_im_name, first_im, medianskip,
                args.medianstep, pdir)
              )
    if args.verbose:
        print(command)
    os.system(command)

    ## Runs peaksearch.py on images using specified thresholds
    command = (
        'peaksearch.py -n %s -F %s -f %i -l %i -o %s/peaks '
        '-d %s/median.edf -p Y --ndigits=%i --OmegaOverRide '
        '-S %f -T %f' %
        (args.stem, args.image_format, first_im, last_im, pdir, pdir, ndigits,
         args.step_ome, args.start_ome)
        )

    ## Adds threshold values onto command
    for t in args.thresholds:
        command +=' -t %s' % t
    if args.verbose:
        print(command)
    os.system(command)

    ## removes any peak entries arising from spots containing less than N pixels
    for t in args.thresholds:
         num_lines = sum(1 for line in open('%s/peaks_t%i.flt' % (pdir, t)))
         if os.path.exists('%s/peaks_t%i.flt' % (pdir, t)) and num_lines > 1:
             d=ic.columnfile('%s/peaks_t%i.flt' % (pdir, t))
             d.removerows('Number_of_pixels', list(range(args.pixels)))
             d.writefile('%s/peaks_min%d_t%i.flt' % (pdir, args.pixels, t))


def main(args):
    ## Define and create directory for saving processed data
    if args.output is None:
        if args.experiment:
            args.output = '%s_foundpeaks' % args.experiment
        else:
            args.output = 'foundpeaks'
    ## Create directory
    if not os.path.exists(args.output):
        os.mkdir(args.output)

    ## Provide default thresholds if not given by user
    if args.thresholds is None:
        args.thresholds=[100,300,500,1000]

    if args.pixels < 1:
        raise RuntimeError('Pixel threshold must be > 0, %s specified' % args.pixels)

    ## Loops through specified number of layers
    for layer in range(args.nlayers):
        if args.verbose:
            print(("\nProcessing layer %d of %d" % (layer+1, args.nlayers)))
        process_layer(args, layer)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        prog='findpeaks',
        description='Finds peaks in area detector image series.',
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog=textwrap.dedent('''
            examples:

             $ findpeaks.py FF_data/a 20 731 171.84 -0.48 -x FF -t 100 -t 300 -t 500 -t 1000
             $ findpeaks.py NF_data/a 20 731 171.36 -0.48 -x NF -t 200 -t 300 -t 500 -t 1000
             $ findpeaks.py FF_data/multigold_ 1 1440 -179.875 0.25 -x FF -t 100 -t 500 -t 1000
             $ findpeaks.py FF_data/ff_ 46 1440 -179.875 0.25 -F .ge2 -x FF -t 100 -t 500 -t 1000
            ''')
        )
    parser.add_argument(
        'stem', type=str, help='image file name stem (path/basename)'
        )
    parser.add_argument(
        'nstart', type=int, help='file number of first image'
        )
    parser.add_argument(
        'nimages', type=int, help='number of images per layer'
        )
    parser.add_argument(
        'start_ome', type=float, help='starting angle'
        )
    parser.add_argument(
        'step_ome', type=float, help='omega step size'
        )
    parser.add_argument(
        '-n', '--nlayers', type=int, default=1, help='number of layers'
        )
    parser.add_argument(
        '-o', '--output', type=str, help='output directory'
        )
    parser.add_argument(
        '-f', '--force', default=False, action='store_true',
        help='force removal of existing layer directory'
        )
    parser.add_argument(
        '-t', '--thresholds', action='append', type=int,
        help='threshold value(s) for peaksearcher'
        )
    parser.add_argument(
        '-p', '--pixels', type=int, default=10,
        help='minimum number of pixels threshold for a peak to be a peak'
        )
    parser.add_argument(
        '-x', '--experiment', type=str, choices=['nf', 'ff', 'NF', 'FF'],
        default='', help='near-field or far-field experiment'
        )
    parser.add_argument(
        '-F', '--image_format', type=str, choices=['.edf', '.tif', '.ge2'],
        default='.tif', help='format of input files'
        )
    parser.add_argument(
        '-m', '--medianstep', type=int, default=10,
        help='add every mth image to generate a median background image'
        )
    parser.add_argument(
        '-v', '--verbose', action='store_true',
        help='report progress in terminal'
        )
    args = parser.parse_args()
    main(args)



from __future__ import print_function

# ImageD11_v0.4 Software for beamline ID11
# Copyright (C) 2005  Jon Wright
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Moved from ImageD11 to xfab 07/11/2019

"""
Class to handle groups of parameters to be saved in and out
of files and edited in guis with the fixed/varied info etc
"""

from xfab import xfab_logging
logger = xfab_logging.get_module_level_logger(__name__)

class par:
    """
    Represents a thing which can vary
    """
    def __init__(self, name, value, helpstring = None ,
                 vary=False, can_vary=False , stepsize = None):
        """
        name : unique key used as keyword arg to some functions
        value : value of the parameter
        helpstring : optional string to help user
        vary : value should be optimised
        can_vary : value is not fixed
        stepsize : guessestimated ball park step size (eg not 1e99!)
        """
        self.name = name
        self.value = value
        if helpstring is None:
            self.helpstring = "parameter : "+name
        else:
            self.helpstring = helpstring
        self.vary = vary
        self.can_vary = can_vary
        self.stepsize = stepsize
        
    def fromstringlist(self, sl):
        """ to send to Java """
        [self.name,
         self.value,
         self.helpstring,
         self.vary,
         self.can_vary,
         self.stepsize] = sl
        
    def tostringlist(self):
        """ to catch from Java """
        return [self.name ,
                self.value ,
                self.helpstring ,
                self.vary ,
                self.can_vary,
                self.stepsize ]

class parameters:
    """
    Class to hold a set of named parameters
    """
    def __init__(self,**kwds):
        """
        name=value style arg list
        """
        self.parameters = kwds
        self.varylist = []
        self.can_vary = {}
        self.variable_list = []
        self.stepsizes = {}
        self.par_objs = {}
        for k,v in list(self.parameters.items()):
            self.addpar(par(k,v))
            
    def addpar(self,par):
        """
        add a parameter object
        """
        self.parameters[par.name] = par.value
        self.can_vary[par.name] = par.can_vary
        if par.vary and par.name not in self.varylist:
            self.varylist.append(par.name)
        if par.can_vary and par.name not in self.variable_list:
            self.variable_list.append(par.name)
            self.stepsizes[par.name]=par.stepsize
        self.par_objs[par.name]=par

    def get_variable_list(self):
        return self.variable_list

    def get_variable_values(self):
        """ values of the parameters """
        return [self.parameters[name] for name in self.varylist]

    def get_variable_stepsizes(self):
        """ stepsizes for optimisers """
        return [self.stepsizes[name] for name in self.varylist]

    def set_varylist(self, vl):
        ks = list(self.parameters.keys())
        for v in vl:
            assert v in ks
            assert v in self.variable_list
        self.varylist = vl

    def set_variable_values(self,values):
        """ set values of the parameters"""
        assert len(values)==len(self.varylist)
        for name, value in zip(self.varylist,values):
            self.parameters[name]=value

    def set_parameters(self,d):
        """
        Updates the values of parameters
        """
        self.parameters.update(d)
        self.dumbtypecheck()

    def get_parameters(self):
        """
        Returns a dictionary of parameters
        """
        return self.parameters

    def get(self,name):
        return self.parameters[name]
    
    def set(self,name,value):
        self.parameters[name] = value

    def update_yourself(self,other):
        """
        Sychronise this parameter objects list of values with another object
        """
        for k,v in list(self.parameters.items()):
            if hasattr(other,k):
                var = getattr(other,k)
                logger.debug("setting: pars[%s] from %s to %s"%(k,v,var))
                self.parameters[k]=var
            else:
                logger.debug("error: %s has no attribute %s, ignoring"%(other,k))

    def update_other(self,other):
        """
        Synchronise an object with the values in this object
        """
        for k,v in list(self.parameters.items()):
            if hasattr(other,k):
                var = getattr(other,k)
                logger.debug("setting: %s.%s from %s to %s"%(other,k,var,v))
                setattr(other,k,v)
            else:
                logger.debug("error: %s has no attribute %s, ignoring"%
                              (other,k))

    def saveparameters(self,filename):
        """
        Write parameters to a file
        """
        f=open(filename,"w")
        keys=list(self.parameters.keys())
        keys.sort()
        for key in keys:
            f.write("%s %s\n"%(key,str(self.parameters[key])))
        f.close()

    def loadparameters(self,filename):
        """
        Load parameters from a file
        """
        lines = open(filename,"r").readlines()
        for line in lines:
            try:
                [name, value] = line.split(" ") 
                name=name.replace("-","_")
                self.parameters[name]=value
            except ValueError:
                logger.error("Failed to read:%s"%(line))
        self.dumbtypecheck()

    def dumbtypecheck(self):
        """
        Eventually parameter types (and units and fixed/varied to be
        specifieable
        For now it just tries to coerce to float, then does nothing
        """
        for name, value in list(self.parameters.items()):
            if type(value) == type("string"):
                try:
                    vf = float(value)
                except ValueError:
                    # it really is a string
                    self.parameters[name] = value.lstrip().rstrip()
                    continue
                # here if float worked
                try:
                    vi = int(value)
                except ValueError:
                    # it really is a float
                    self.parameters[name] = vf
                    continue
                    
                # here if float and int worked
                # should not be needed, depends on int valueerror
                if abs(vi - vf) < 1e-9:
                    # use int
                    self.parameters[name] = vi
                    continue
                else:
                    self.parameters[name] = vf
                    continue
            else:
                # int/float preserve type
                self.parameters[name] = value

def read_par_file( filename ):
    p = parameters()
    p.loadparameters( filename )
    return p



"""
xfab is a library of data and functions for use in Crystallographic computations. xfab is part of the project fable: http://fable.wiki.sourceforge.net
"""


"""Set a package wide global variable that toogle input/output on functions.
Checks are on by default. Toogling of checks can be achived as:

    import xfab
    xfab.CHECKS.activated = False

"""
from xfab.checks import _checkState
CHECKS = _checkState()



#!D:\bld\xfab_1644315860030\_h_env\python.exe

# Create objects to manipulate - they hold your data
#
from __future__ import absolute_import
import sys
from ImageD11 import peakmerge, indexing, transformer
mypeakmerger = peakmerge.peakmerger()
mytransformer = transformer.transformer()
myindexer = indexing.indexer()
#
# Your work starts here:
#
filename = sys.argv[1]
outfile =  sys.argv[2]
parfile =  sys.argv[3]
mytransformer.loadfiltered(  filename )
mytransformer.loadfileparameters( parfile )
mytransformer.compute_tth_eta( )
mytransformer.getcolumn(  'tth' )
mytransformer.getcolumn(  'eta' )
mytransformer.addcellpeaks( )
mytransformer.computegv( )
mytransformer.savegv(  outfile )



#!D:\bld\xfab_1644315860030\_h_env\python.exe



##########################################################################
# IMPORT PYTHON PACKAGES
from __future__ import absolute_import
from __future__ import print_function
import os
import argparse
import textwrap
from six.moves import range

##########################################################################
def sortFLT(FLTfile,newFLTfile,args):

    f=open(FLTfile,'r')
    fw=open(newFLTfile,'w')

    for i in range(2):
        line=f.readline()
        fw.write(line)
    titles = line.split()
   
    eof = 0
    while not(eof):
        line = f.readline()
        s = line.split()
        if len(s)>0:
       
            if float(s[1]) < args.bmaxx and float(s[1]) > args.bminx and float(s[0]) < args.bmaxy and float(s[0]) > args.bminy:
                a = 11
            else:
                fw.write('%s' % line)

        else:
            eof = 1

    f.close()
    fw.close()
    
    print('\nCreated file %s.' % (newFLTfile))


    return()

##########################################################################
def main(args):

    if not os.path.exists(args.fltfile):
    	raise IOError('File %s does not exist.' % args.fltfile)

    newfltfile =  '%s_%s.flt' % (args.fltfile.split('.')[0],'noBEAM')
    if os.path.exists(newfltfile) and not args.force:
    	raise IOError('File %s already exists.' % newfltfile)


    if args.bmaxx < args.bminx:
        newmax = args.bminx
        args.bminx = args.bmaxx
        args.bmaxx = newmax
        if args.verbose:
            print('Redefined limits of beamspot: x min and max were switched.')
    if args.bmaxy < args.bminy:
        newmax = args.bminy
        args.bminy = args.bmaxy
        args.bmaxy = newmax
        if args.verbose:
            print('Redefined limits of beamspot: y min and max were switched.')

    sortFLT(args.fltfile,newfltfile,args)

    return()

##########################################################################
if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        prog='flt_remove_beam.py',
        description='Removes peaks in within the near-field beam spot from flt file.',
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog=textwrap.dedent('''
             examples:
 
             $ 
             $ flt_remove_beam.py peaks_t100.flt 250 1900 935 1125
             ''')
         )

    parser.add_argument(
        'fltfile', type=str, help='flt file'
        )
    parser.add_argument(
        'bminx', type=int, help='beam spot limits: min. x'
        )        
    parser.add_argument(
        'bmaxx', type=int, help='beam spot limits: max. x'
        )        
    parser.add_argument(
        'bminy', type=int, help='beam spot limits: min. y'
        )        
    parser.add_argument(
        'bmaxy', type=int, help='beam spot limits: max. y'
        )        
    parser.add_argument(
        '-f', '--force', default=False, action='store_true',
        help='force removal of report file for overwriting'
        )  
    parser.add_argument(
        '-v', '--verbose', action='store_true',
        help='report progress in terminal'
        )
    args = parser.parse_args()
    main(args)




"""The checks module implements input checks for various xfab functions. These checks
are meant to be called upon during runtime to catch errors made on the user side. To
toogle checks in xfab a package wide constant CHECKS defined in the __init__.py should
be accessed as:

import xfab
xfab.CHECKS.activated = False

"""
import numpy as np

def _check_rotation_matrix(U):
    """Verify that a 3 x 3 matrix is a rotation matrix.

    Args:
        U: 3x3 matrix represented as a numpy array of shape=(3,3).

    """
    if not np.allclose( np.dot(U.T, U), np.eye(3,3) ):
        raise ValueError("orientation matrix U is not unitary, np.dot(U.T, U)!=np.eye(3,3)")

    if not np.allclose( np.linalg.det(U), 1.0 ):
        raise ValueError("orientation matrix U has a non unity determinant np.linalg.det(U)!=1.0")

def _check_euler_angles(phi1, PHI, phi2):
    """Verify that all three Euler angles lies in the range [0,2*pi].

    Args:
        phi1, PHI, and phi2: Euler angles in radians.

    """
    if not (0<=phi1<=np.pi*2):
        raise ValueError("Euler angle phi1="+str(phi1)+" is not in range [0,2*pi]")

    if not (0<=PHI<=np.pi*2):
        raise ValueError("Euler angle PHI="+str(PHI)+" is not in range [0,2*pi]")

    if not (0<=phi2<=np.pi*2):
        raise ValueError("Euler angle phi2="+str(phi2)+" is not in range [0,2*pi]")

class _checkState(object):

    """A checkState object can be used to determine if checks are active in xfab
        The default mode is active and can be togled. This allows users to in some
        specific scenarios speed up their codes. Furthermore, if __debug__==False
        all checks are inactivated, i.e using python -O to run codes will remove 
        any checks.
    """
    def __init__(self):
        self._run_checks = True

    @property
    def activated(self):
        """True if checks are to be run.
        """
        return self._run_checks and __debug__

    @activated.setter
    def activated(self, value):
        if value is not True and value is not False:
            raise ValueError("Please supply a boolean True or False")
        else:
            self._run_checks = value


"""
xfab.structure for reading crystal structure files (cif,pdb)
and calculation of form factors and structure factors etc. 
"""

from __future__ import absolute_import
import numpy as n

from xfab import tools
from xfab import sg
from xfab import atomlib
from six.moves import range

from xfab import xfab_logging
logger = xfab_logging.get_module_level_logger(__name__)

def StructureFactor(hkl, ucell, sgname, atoms, disper = None):
    """
    Calculation of the structure factor of reflection hkl
    
    [Freal Fimg] = StructureFactor(hkl,unit_cell,sg,atoms)
    
    INPUT : hkl =       [h, k, l] 
            unit_cell = [a, b, c, alpha, beta, gamma] 
            sgname:     space group name (e.g. 'P 21/c')
            atoms:      structural parameters (as an object)
    OUTPUT: The real and imaginary parts of the the structure factor
    
    Henning Osholm Sorensen, June 23, 2006.
    Translated to python code April 8, 2008
    """
    mysg = sg.sg(sgname = sgname) 
    stl = tools.sintl(ucell, hkl)
    noatoms = len(atoms)

    Freal = 0.0
    Fimg = 0.0

    for i in range(noatoms):
        #Check whether isotrop or anisotropic displacements 
        if atoms[i].adp_type == 'Uiso':
            U = atoms[i].adp
            expij = n.exp(-8*n.pi**2*U*stl**2)
        elif atoms[i].adp_type == 'Uani':
            # transform Uij to betaij
            betaij = Uij2betaij(atoms[i].adp, ucell)
        else:
            expij = 1
            atoms[i].adp = 'Uiso'
            #logger.error("wrong no of elements in atomlist")

        # Atomic form factors
        f = FormFactor(atoms[i].atomtype, stl)
        if disper == None or disper[atoms[i].atomtype] == None :
            fp = 0.0
            fpp = 0.0
        else:
            fp = disper[atoms[i].atomtype][0]
            fpp = disper[atoms[i].atomtype][1]

        for j in range(mysg.nsymop):
            # atomic displacement factor
            if atoms[i].adp_type == 'Uani':
                betaijrot = n.dot(mysg.rot[j], n.dot(betaij, mysg.rot[j]))
                expij = n.exp(-n.dot(hkl, n.dot(betaijrot, hkl)))
                
            # exponent for phase factor
            r = n.dot(mysg.rot[j], atoms[i].pos) + mysg.trans[j]
            exponent = 2*n.pi*n.dot(hkl, r)

            #forming the real and imaginary parts of F
            s = n.sin(exponent)
            c = n.cos(exponent)
            site_pop = atoms[i].occ*atoms[i].symmulti/mysg.nsymop
            Freal = Freal + expij*(c*(f+fp)-s*fpp)*site_pop
            Fimg = Fimg + expij*(s*(f+fp)+c*fpp)*site_pop

    return [Freal, Fimg]

def Uij2betaij(adp, ucell):
    """
    Uij2betaij transform the ADP U-matrix into the beta form 
    
    betaij = Uij2betaij(adp,unit_cell)
    
    INPUT:  adp: anisotropic displacement parameter U matrix
            Uijs are given in this order: [U11, U22, U33, U23, U13, U12]
            unit_cell = [a, b, c, alpha, beta, gamma] 

    OUTPUT: betaij: beta displacement matrix
    
    Henning Osholm Sorensen, Risoe National Laboratory, June 23, 2006.
    Translated to python code March 29, 2008
    """
    U  = n.array([[adp[0], adp[5], adp[4]],
                  [adp[5], adp[1], adp[3]], 
                  [adp[4], adp[3], adp[2]]])

    betaij = n.zeros((3, 3))
    cellstar = tools.cell_invert(ucell)
 
    for i in range(3):
        for j in range(3):
            betaij[i, j] = 2*n.pi**2*cellstar[i]*cellstar[j]*U[i, j]

    return betaij



def FormFactor(atomtype, stl):
    """
     Calculation of the atomic form factor at a specified sin(theta)/lambda
     using the analytic fit to the  form factors from 
     Int. Tab. Cryst Sect. C 6.1.1.4

     INPUT:   atomtype: Atom type (string) e.g. 'C' 
              stl: form factor calculated sin(theta)/lambda = stl
     OUTPUT:  atomic form factor (no dispersion)
              

     Henning Osholm Sorensen, Risoe National Laboratory, April 9, 2008.

    """

    data = atomlib.formfactor[atomtype]

    # Calc form factor
    formfac = 0

    for i in range(4):
        formfac = formfac + data[i]*n.exp(-data[i+4]*stl*stl) 
    formfac = formfac + data[8]

    return formfac


def int_intensity(F2, L, P, I0, wavelength, cell_vol, cryst_vol):
    """
    Calculate the reflection intensities scaling factor
    
    INPUT:
    F2        : the structure factor squared
    L         : Lorentz factor
    P         : Polarisation factor
    I0        : Incoming beam flux
    wavelength: in Angstroem
    cell_vol  : Volume of unit cell in AA^3
    cryst_vol : Volume of crystal in mm^3

    OUTPUT:
    int_intensity: integrated intensity

    """
#    print F2,L,P,I0,wavelength,cell_vol,cryst_vol
    
    emass = 9.1093826e-31
    echarge = 1.60217653e-19
    pi4eps0 = 1.11265e-10
    c = 299792458.0
    k1 = (echarge**2/(pi4eps0*emass*c**2)*1000)**2 # Unit is mm
    # the factor 1e21 used below is to go from mm^3 to AA^3
    k2 = wavelength**3 * cryst_vol * 1e21/cell_vol**2 
    return k1*k2*I0*L*P*F2


def multiplicity(position, sgname=None, sgno=None, cell_choice='standard'):
    """
    Calculates the multiplicity of a fractional position in the unit cell.
    If called by sgno, cell_choice is necessary for eg rhombohedral space groups.

    """

    if sgname != None:
        mysg = sg.sg(sgname=sgname, cell_choice=cell_choice)
    elif sgno !=None:
        mysg = sg.sg(sgno=sgno, cell_choice=cell_choice)
    else:
        raise ValueError('No space group information provided')

    lp = n.zeros((mysg.nsymop, 3))

    for i in range(mysg.nsymop):
        lp[i, :] = n.dot(position, mysg.rot[i]) + mysg.trans[i]

    lpu = n.array([lp[0, :]])
    multi = 1

    for i in range(1, mysg.nsymop):
        for j in range(multi):
            t = lp[i]-lpu[j]
            if n.sum(n.mod(t, 1)) < 0.00001:
                break
            else:
                if j == multi-1:
                    lpu = n.concatenate((lpu, [lp[i, :]]))
                    multi += 1
    return multi

class atom_entry:
    def __init__(self, label=None, atomtype=None, pos=None,
                 adp_type=None, adp=None, occ=None, symmulti=None):
        self.label = label
        self.atomtype = atomtype
        self.pos = pos
        self.adp_type = adp_type
        self.adp = adp
        self.occ = occ
        self.symmulti = symmulti

class atomlist:
    def __init__(self, sgname=None, sgno=None, cell=None):
        self.sgname = sgname
        self.sgno = sgno
        self.cell = cell
        self.dispersion = {}
        self.atom = []
    def add_atom(self, label=None, atomtype=None, pos=None, 
                 adp_type=None, adp=None, occ=None, symmulti=None):
        self.atom.append(atom_entry(label=label, atomtype=atomtype,
                                    pos=pos, adp_type=adp_type,
                                    adp=adp, occ=occ, symmulti=symmulti))

class build_atomlist:
    def __init__(self):
        self.atomlist = atomlist()
        
    def CIFopen(self, ciffile=None, cifblkname=None):
        from CifFile import ReadCif # part of the PycifRW module
        try:
            cf = ReadCif(ciffile)
        except:
            logger.error('File %s could not be accessed' %ciffile)        

        if cifblkname == None:   
            #Try to guess blockname                                                     
            blocks = list(cf.keys())
            if len(blocks) > 1:
                if len(blocks) == 2 and 'global' in blocks:
                    cifblkname = blocks[abs(blocks.index('global') - 1)]
                else:
                    logger.error('More than one possible data set:')
                    logger.error('The following data block names are in the file:')
                    for block in blocks:
                        logger.error(block)
                    raise Exception
            else:
                # Only one available
                cifblkname = blocks[0]
        #Extract block
        try:
            self.cifblk = cf[cifblkname]
        except:
            logger.error('Block - %s - not found in %s' % (cifblkname, ciffile))
            raise IOError
        return self.cifblk

    def PDBread(self, pdbfile = None):
        """
        function to read pdb file (www.pdb.org) and make 
        atomlist structure
        """
        from re import sub
        try:
            text = open(pdbfile, 'r').readlines()
        except:
            logger.error('File %s could not be accessed' % pdbfile)


        for i in range(len(text)):
            if text[i].find('CRYST1') == 0:
                a = float(text[i][6:15])
                b = float(text[i][15:24])
                c = float(text[i][24:33])
                alp = float(text[i][33:40])
                bet = float(text[i][40:47])
                gam = float(text[i][47:54])
                sg = text[i][55:66]

        self.atomlist.cell = [a, b, c, alp, bet, gam]

        # Make space group name
        sgtmp = sg.split()
        sg = ''
        for i in range(len(sgtmp)):
            if sgtmp[i] != '1':
                sg = sg + sgtmp[i].lower()
        self.atomlist.sgname = sg

        # Build SCALE matrix for transformation of 
        # orthonormal atomic coordinates to fractional
        scalemat = n.zeros((3, 4))
        for i in range(len(text)):
            if text[i].find('SCALE') == 0:
                # FOUND SCALE LINE
                scale = text[i].split()
                scaleline = int(scale[0][-1])-1
                for j in range(1, len(scale)):
                    scalemat[scaleline, j-1] = float(scale[j])
                
        no = 0
        for i in range(len(text)):
            if text[i].find('ATOM') == 0 or text[i].find('HETATM') ==0:
                no = no + 1 
                label = sub("\s+", "", text[i][12:16])
                atomtype = sub("\s+", "", text[i][76:78]).upper()
                x = float(text[i][30:38])
                y = float(text[i][38:46])
                z = float(text[i][46:54])
                # transform orthonormal coordinates to fractional
                pos = n.dot(scalemat, [x, y, z, 1])
                adp = float(text[i][60:66])/(8*n.pi**2) # B to U
                adp_type = 'Uiso'
                occ = float(text[i][54:60])
                multi = multiplicity(pos, self.atomlist.sgname)
                self.atomlist.add_atom(label=label,
                                       atomtype=atomtype,
                                       pos = pos,
                                       adp_type= adp_type,
                                       adp = adp,
                                       occ=occ,
                                       symmulti=multi)

                self.atomlist.dispersion[atomtype] = None


    def CIFread(self, ciffile = None, cifblkname = None, cifblk = None):
        from re import sub
        if ciffile != None:
            try:
                cifblk = self.CIFopen(ciffile=ciffile, cifblkname=cifblkname)
            except:
                raise 
        elif cifblk == None:
            cifblk = self.cifblk

        self.atomlist.cell = [self.remove_esd(cifblk['_cell_length_a']),
                              self.remove_esd(cifblk['_cell_length_b']),
                              self.remove_esd(cifblk['_cell_length_c']),
                              self.remove_esd(cifblk['_cell_angle_alpha']),
                              self.remove_esd(cifblk['_cell_angle_beta']),
                              self.remove_esd(cifblk['_cell_angle_gamma'])]

        #self.atomlist.sgname = upper(sub("\s+","",
        #                       cifblk['_symmetry_space_group_name_H-M']))
        self.atomlist.sgname = sub("\s+", "",
                                   cifblk['_symmetry_space_group_name_H-M'])

        # Dispersion factors
        if '_atom_type_symbol' in list(cifblk.keys()):
            for i in range(len(cifblk['_atom_type_symbol'])):
                try:
                    self.atomlist.dispersion[cifblk['_atom_type_symbol'][i].upper()] =\
                        [self.remove_esd(cifblk['_atom_type_scat_dispersion_real'][i]),
                        self.remove_esd(cifblk['_atom_type_scat_dispersion_imag'][i])]
                except:
                    self.atomlist.dispersion[cifblk['_atom_type_symbol'][i].upper()] = None
                    logger.warning('No dispersion factors for %s in cif file - set to zero'\
                                        %cifblk['_atom_type_symbol'][i])
        else:
            logger.warning('No _atom_type_symbol found in CIF')
            for i in range(len(cifblk['_atom_site_type_symbol'])):
                self.atomlist.dispersion[cifblk['_atom_site_type_symbol'][i].upper()] = None
                logger.warning('No dispersion factors for %s in cif file - set to zero'\
                                        %cifblk['_atom_site_type_symbol'][i])

        for i in range(len(cifblk['_atom_site_type_symbol'])):
            label = cifblk['_atom_site_label'][i]
            #atomno = atomtype[upper(cifblk['_atom_site_type_symbol'][i])]
            atomtype = cifblk['_atom_site_type_symbol'][i].upper()
            x = self.remove_esd(cifblk['_atom_site_fract_x'][i])
            y = self.remove_esd(cifblk['_atom_site_fract_y'][i])
            z = self.remove_esd(cifblk['_atom_site_fract_z'][i])
            try:
                adp_type = cifblk['_atom_site_adp_type'][i]
            except:
                adp_type = None
            try:
                occ = self.remove_esd(cifblk['_atom_site_occupancy'][i])
            except:
                occ = 1.0

            if '_atom_site_symmetry_multiplicity' in cifblk:
                multi = self.remove_esd(cifblk['_atom_site_symmetry_multiplicity'][i])
            # In old SHELXL versions this code was written
            # as '_atom_site_symetry_multiplicity'
            elif '_atom_site_symetry_multiplicity' in cifblk:
                multi = self.remove_esd(cifblk['_atom_site_symetry_multiplicity'][i])
            else:
                multi = multiplicity([x, y, z], self.atomlist.sgname)

            # Test for B or U

            if adp_type == None:
                adp = 0.0
            elif adp_type == 'Biso':
                adp = self.remove_esd(cifblk['_atom_site_B_iso_or_equiv'][i])/(8*n.pi**2)
                adp_type = 'Uiso'
            elif adp_type == 'Bani':
                anisonumber = cifblk['_atom_site_aniso_label'].index(label)
                adp = [ self.remove_esd(cifblk['_atom_site_aniso_B_11'][anisonumber])/(8*n.pi**2),
                        self.remove_esd(cifblk['_atom_site_aniso_B_22'][anisonumber])/(8*n.pi**2),
                        self.remove_esd(cifblk['_atom_site_aniso_B_33'][anisonumber])/(8*n.pi**2),
                        self.remove_esd(cifblk['_atom_site_aniso_B_23'][anisonumber])/(8*n.pi**2),
                        self.remove_esd(cifblk['_atom_site_aniso_B_13'][anisonumber])/(8*n.pi**2),
                        self.remove_esd(cifblk['_atom_site_aniso_B_12'][anisonumber])/(8*n.pi**2)]
                adp_type = 'Uani'
            elif adp_type == 'Uiso':
                adp = self.remove_esd(cifblk['_atom_site_U_iso_or_equiv'][i])
            elif adp_type == 'Uani':
                anisonumber = cifblk['_atom_site_aniso_label'].index(label)
                adp = [ self.remove_esd(cifblk['_atom_site_aniso_U_11'][anisonumber]),
                        self.remove_esd(cifblk['_atom_site_aniso_U_22'][anisonumber]),
                        self.remove_esd(cifblk['_atom_site_aniso_U_33'][anisonumber]),
                        self.remove_esd(cifblk['_atom_site_aniso_U_23'][anisonumber]),
                        self.remove_esd(cifblk['_atom_site_aniso_U_13'][anisonumber]),
                        self.remove_esd(cifblk['_atom_site_aniso_U_12'][anisonumber])]
            self.atomlist.add_atom(label=label, atomtype=atomtype,
                                   pos = [x, y, z], adp_type= adp_type, 
                                   adp = adp, occ=occ , symmulti=multi)

    def remove_esd(self, a):
        """                                                                         
        This function will remove the esd part of the entry,
        e.g. '1.234(56)' to '1.234'.
        """

        
        if a.find('(') == -1:
            value = float(a)
        else:
            value = float(a[:a.find('(')])
        return value





#!D:\bld\xfab_1644315860030\_h_env\python.exe

from __future__ import absolute_import
from __future__ import print_function
from optparse import OptionParser
import sys
import numpy as np
import pylab as pl
from xfab import symmetry,tools
#import matplotlib.axes3d as p3
import mpl_toolkits.mplot3d as p3
from ImageD11 import columnfile as ic
from matplotlib import cm,colors
from six.moves import range

"""
For plotting gff files with colour coding according to orientation, stress or strain.
Options for controlling labels, viewing angle, plot and grain size.
Two gffs can be plotted simultaneously (eg for two-phase materials)

Jette Oddershede, DTU Physics, March 2013 (jeto@fysik.dtu.dk)

Fix for hexagonal inverse pole figure, April 2015 (mkak)
"""


if __name__=='__main__':


  def get_options():
    parser = OptionParser()
    parser.add_option("-i", "--input", action="store",
                      dest="input", type="string",
                      help="stem of input .gff")
    parser.add_option("-s", "--scale", action="store",
                      dest="scale", type="float",
                      default = None, #Do not scale grain sizes
                      help="Scale factor for grain sizes, default: 1")
    parser.add_option("-d", "--dim", action="store",
                      dest="dim", type="float",
                      default = 1, #plot -1<x,y,z<1
                      help="Plot size in mm, default: 1")
    parser.add_option("-t", "--text", action="store",
                      dest="text", type="string",
                      default = None, #do not give grain numbers on plot
                      help="Give grain numbers on plot (yes/no), default: None")
    parser.add_option("-c", "--sym", action="store",
                      dest="sym", type="string",
                      default = None, #no symmetry information
                      help="Colour according to symmetry information, options: \
                      None (default, random colours) \
                      standard (allowing all orientations in cubic space) \
                      cubic (according to cubic inverse pole figure - IPF) \
                      hexagonal (according to hexagonal inverse pole figure) \
                      orthorhombic (according to orthorhombic IPF) \
                      s11/s22/s33/s23/s13/s12 (stress tensor components) \
                      e11/e22/e33/e23/e13/e12 (strain tensor components)")
    parser.add_option("-m", "--min", action="store",
                      dest="minimum", type="int",
                      default=None, #Use parameter-specific defaults
                      help="Give minimum for colour bar      \
                      Must be integer, use default: None to seen unit")
    parser.add_option("-M", "--max", action="store",
                      dest="maximum", type="int",
                      default=None, #Use parameter-specific defaults
                      help="Give maximum for colour bar      \
                      Must be integer, use default: None to seen unit")
    parser.add_option("-p", "--step", action="store",
                      dest="step", type="int",
                      default=None, #Use parameter-specific defaults
                      help="Give step size for colour bar      \
                      Must be integer, use default: None to seen unit")
    parser.add_option("-v", "--view", action="store",
                      dest="view", type="string",
                      default=None, help="View structure along the x/y/z axis")
    parser.add_option("-I", "--input2", action="store",
                      dest="input2", type="string",
                      default= None, help="stem of second input .gff")
    parser.add_option("-S", "--scale2", action="store",
                      dest="scale2", type="float",
                      default = None, #Do not scale grain sizes
                      help="Scale factor for grain sizes of file2, default: 1")
    parser.add_option("-T", "--text2", action="store",
                      dest="text2", type="string",
                      default = None, #do not give grain numbers on plot
                      help="Give grain numbers on plot for file2 (yes/no), default: None")
    parser.add_option("-C", "--sym2", action="store",
                      dest="sym2", type="string",
                      default = None, #no symmetry information
                      help="Colour according to symmetry information of file2         (see options for -c)")



    options , args = parser.parse_args()

    do_exit = False

    if options.input == None:
      print("\nNo stem of input .gff supplied [-i input]\n")
      do_exit = True
    if do_exit:
        parser.print_help()
        sys.exit()
    return options

    
  def read(file,sym,minimum,maximum,step):
    data = ic.columnfile('%s.gff' %file)
    #grain sizes
    if "grainno" not in data.titles:
        data.addcolumn(data.grain_id,'grainno')
    if "grainvolume" in data.titles:
        scale = max(data.grainvolume**0.3333)
        data.addcolumn(data.grainvolume**0.3333/scale*100,'size')
    else:
        data.addcolumn(100.*np.ones(data.nrows),'size')
    rodx = []
    rody = []
    rodz = []
    if "rodx" not in data.titles:
        for i in range(data.nrows):
            U = np.array([[data.U11[i],data.U12[i],data.U13[i]],
                         [data.U21[i],data.U22[i],data.U23[i]],
                         [data.U31[i],data.U32[i],data.U33[i]]])
            rod = tools.u_to_rod(U)
            rodx.append(rod[0])
            rody.append(rod[1])
            rodz.append(rod[2])
        data.addcolumn(np.array(rodx),'rodx')
        data.addcolumn(np.array(rody),'rody')
        data.addcolumn(np.array(rodz),'rodz')
    if sym == "standard":
        #grain colours, so that all orientations in Cubic space are allowed
        maxhx=62.8*3.14/180 
        minhx=-62.8*3.14/180
        maxhy=62.8*3.14/180 
        minhy=-62.8*3.14/180
        maxhz=62.8*3.14/180 
        minhz=-62.8*3.14/180; 
        rr = data.rodx**2+data.rody**2+data.rodz**2
        rr = np.sqrt(rr)
        theta=2*np.arctan(rr)
        r1=data.rodx/rr 
        r2=data.rody/rr 
        r3=data.rodz/rr
        # normalise colours
        red =   (r1*theta-minhx)/(maxhx-minhx)
        green = (r2*theta-minhy)/(maxhy-minhy)
        blue =  (r3*theta-minhz)/(maxhz-minhz)
    elif sym == "orthorhombic":
        # Fill orthorhombic stereographic triangle
        red = []
        green = []
        blue = []
        fig = pl.figure(10,frameon=False,figsize=pl.figaspect(1.0))
        ax = pl.Axes(fig,[.2,.2,.7,.7])
        ax.set_axis_off()
        fig.add_axes(ax)
        #plot triangle    
        xa = np.zeros((101))
        ya = np.zeros((101))
        for i in range(101):
            xa[i] = np.cos(i*np.pi/200.)
            ya[i] = np.sin(i*np.pi/200.)
        pl.plot(xa,ya,'black') # Curved edge
        pl.plot([0,1],[0,0],'black',linewidth=2) #lower line 
        pl.plot([0,0],[1,0],'black',linewidth=2) #left line 
        pl.text(-0.02,-0.04,'[001]')
        pl.text(0.95,-0.04,'[100]')
        pl.text(-0.02,1.01,'[010]')
        # Grains
        for i in range(data.nrows):
            U = tools.rod_to_u([data.rodx[i],data.rody[i],data.rodz[i]])
            axis = abs(U[2,:])
            colour = np.zeros((3))
            colour[0]=(2*np.arcsin(abs(axis[2]))/np.pi)**1; 
            colour[1]=(2*np.arcsin(abs(axis[0]))/np.pi)**1;
            colour[2]=(2*np.arcsin(abs(axis[1]))/np.pi)**1;
            mx = max(colour)
            colour = colour/mx
            red.append(colour[0])
            green.append(colour[1])
            blue.append(colour[2])
            X = axis[0]/(1+axis[2])
            Y = axis[1]/(1+axis[2])            
            pl.plot(X,Y,'o',color=colour)
        pl.xlim()
    elif sym == "cubic":
        # Fill cubic stereographic triangle
        red = []
        green = []
        blue = []
        fig = pl.figure(10,frameon=False,figsize=pl.figaspect(.9))
        ax = pl.Axes(fig,[.2,.2,.7,.7])
        ax.set_axis_off()
        fig.add_axes(ax)
        #plot triangle    
        xa = np.zeros((21))
        ya = np.zeros((21))
        for i in range(21):
            ua = np.array([i/20., 1., 1.])
            UA = np.linalg.norm(ua)
            za = ua[2]+UA
            xa[i] = ua[1]/za
            ya[i] = ua[0]/za
        pl.plot(xa,ya,'black') # Curved edge
        pl.plot([0,xa[0]],[0,0.0001],'black',linewidth=2) #lower line 
        pl.plot([xa[20],0],[ya[20],0],'black') # upper line
        pl.text(-0.01,-0.02,'[100]')
        pl.text(xa[0]-0.01,-0.02,'[110]')
        pl.text(xa[-1]-0.01,ya[-1]+0.005,'[111]')
        # Grains
        for i in range(data.nrows):
            U = tools.rod_to_u([data.rodx[i],data.rody[i],data.rodz[i]])
            axis = abs(U[2,:])
            colour = np.zeros((3))
            for j in range(3):
                for k in range(j+1,3):
                    if (axis[j]>axis[k]):
                        colour[0]=axis[j]
                        axis[j]=axis[k]
                        axis[k]=colour[0]                 
            rr=np.sqrt(axis[0]*axis[0]/((axis[2]+1))/((axis[2]+1))+(axis[1]/(axis[2]+1)+1)*(axis[1]/(axis[2]+1)+1))
            if axis[1]==0: 
                beta=0
            else:
                beta=np.arctan(axis[0]/axis[1])
            colour[0]=((np.sqrt(2.0)-rr)/(np.sqrt(2.0)-1))**.5;
            colour[1]=((1-4*beta/np.pi)*((rr-1)/(np.sqrt(2.0)-1)))**.5;
            colour[2]=(4*beta/np.pi*((rr-1)/(np.sqrt(2.0)-1)))**.5;
            mx = max(colour)
            colour = colour/mx
            red.append(colour[0])
            green.append(colour[1])
            blue.append(colour[2])
            X = axis[1]/(1+axis[2])
            Y = axis[0]/(1+axis[2])            
            pl.plot(X,Y,'o',color=colour)
        pl.xlim()
    elif sym == "hexagonal":

        ## Fill hexagonal stereographic triangle
        A = np.array([[ 0,           1, 0],
                      [ 0, -np.sqrt(3), 1],
                      [ 1,           0, 0]])
        
        a0 = 1./np.sqrt(3.)
        a1 = 1.
        a2 = 1.

        red = []
        green = []
        blue = []

        fig = pl.figure(10,frameon=False,figsize=pl.figaspect(0.5))
        ax = pl.Axes(fig,[0.2,.2,0.6,0.6])
        ax.set_axis_off()
        fig.add_axes(ax)

        ## Plot triangle    
        ya = np.array(list(range(51)))/100.
        xa = np.sqrt(1-ya**2)
        pl.plot(xa,ya,'black') # Curved edge
        pl.plot([0,xa[0]],[0,0.0001],'black',linewidth=2) #lower line 
        pl.plot([xa[-1],0],[ya[-1],0],'black') # upper line
        
        ## Label crystalographic directions 
        pl.text(-0.01,-0.02,'[0001]')
        pl.text(xa[0]-0.03,-0.02,'[2-1-10]')
        pl.text(xa[-1]-0.03,ya[-1]+0.005,'[10-10]')

        ## Grains
        r = symmetry.rotations(6)
        
        for i in range(data.nrows):

            U = tools.rod_to_u([data.rodx[i],data.rody[i],data.rodz[i]])

            square = 1
            angle = 0
            frac = 1./np.sqrt(3.)

            for k in range(len(r)):

                g = np.dot(U,r[k])
                a = np.arccos((np.trace(g)-1)/2)                

                if g[2,2] > 0:
                    uvw = g[2,:]
                else:
                    uvw = -g[2,:]
                    
                ## needed to switch these indices to get correct color and inv pf location
                switch1 = uvw[0]
                switch2 = uvw[1]
                uvw[0] = switch2
                uvw[1] = switch1

                x = uvw[0]/(1+uvw[2])
                y = uvw[1]/(1+uvw[2])
                
                f = y/x
                s = x*x+y*y
                    
                ## Finds r (symmetry) which plots grain into triangle
                if f<=frac and s<=square and x>=0 and y>=0:
                    angle = a
                    frac = f
                    square = s
                    UVW = uvw
                    X = x
                    Y = y
            
            colour = np.dot(np.transpose(A),np.transpose(UVW))**0.7

            colour[0] = colour[0]/a2
            colour[1] = colour[1]/a1
            colour[2] = colour[2]/a0
            mx = max(colour)
            colour = colour/mx
           
            red.append(colour[0])
            green.append(colour[1])
            blue.append(colour[2])

            pl.plot(X,Y,'o',color=colour)
        pl.xlim()
    elif sym == "e11":
        try:
            colourbar(minimum,maximum,step,'e-3')
            norm = colors.Normalize(minimum*1e-3,maximum*1e-3)
        except:
            colourbar(-1,1,1,'e-3')
            norm = colors.Normalize(-1e-3,1e-3)
        color = cm.jet(norm(data.eps11_s))
        red = color[:,0]
        green = color[:,1]
        blue = color[:,2]
        print("transverse strain:", np.sum(data.grainvolume*data.eps11_s)/np.sum(data.grainvolume))
    elif sym == "e22":
        try:
            colourbar(minimum,maximum,step,'e-3')
            norm = colors.Normalize(minimum*1e-3,maximum*1e-3)
        except:
            colourbar(-1,1,1,'e-3')
            norm = colors.Normalize(-1e-3,1e-3)
        color = cm.jet(norm(data.eps22_s))
        red = color[:,0]
        green = color[:,1]
        blue = color[:,2]
        print("transverse strain:", np.sum(data.grainvolume*data.eps22_s)/np.sum(data.grainvolume))
    elif sym == "e33":
        try:
            colourbar(minimum,maximum,step,'e-3')
            norm = colors.Normalize(minimum*1e-3,maximum*1e-3)
        except:
            colourbar(-1,1,1,'e-3')
            norm = colors.Normalize(-1e-3,1e-3)
        color = cm.jet(norm(data.eps33_s))
        red = color[:,0]
        green = color[:,1]
        blue = color[:,2]
        print("axial strain:", np.sum(data.grainvolume*data.eps33_s)/np.sum(data.grainvolume))
    elif sym == "e12":
        try:
            colourbar(minimum,maximum,step,'e-3')
            norm = colors.Normalize(minimum*1e-3,maximum*1e-3)
        except:
            colourbar(-1,1,1,'e-3')
            norm = colors.Normalize(-1e-3,1e-3)
        color = cm.jet(norm(data.eps12_s))
        red = color[:,0]
        green = color[:,1]
        blue = color[:,2]
        print("shear strain:", np.sum(data.grainvolume*data.eps12_s)/np.sum(data.grainvolume))
    elif sym == "e13":
        try:
            colourbar(minimum,maximum,step,'e-3')
            norm = colors.Normalize(minimum*1e-3,maximum*1e-3)
        except:
            colourbar(-1,1,1,'e-3')
            norm = colors.Normalize(-1e-3,1e-3)
        color = cm.jet(norm(data.eps13_s))
        red = color[:,0]
        green = color[:,1]
        blue = color[:,2]
        print("shear strain:", np.sum(data.grainvolume*data.eps13_s)/np.sum(data.grainvolume))
    elif sym == "e23":
        try:
            colourbar(minimum,maximum,step,'e-3')
            norm = colors.Normalize(minimum*1e-3,maximum*1e-3)
        except:
            colourbar(-1,1,1,'e-3')
            norm = colors.Normalize(-1e-3,1e-3)
        color = cm.jet(norm(data.eps23_s))
        red = color[:,0]
        green = color[:,1]
        blue = color[:,2]
        print("shear strain:", np.sum(data.grainvolume*data.eps23_s)/np.sum(data.grainvolume))
    elif sym == "s33":
        try:
            colourbar(minimum,maximum,step,'MPa')
            norm = colors.Normalize(minimum,maximum)
        except:
            colourbar(-50,150,50,'MPa')
            norm = colors.Normalize(-50,150)
        color = cm.jet(norm(data.sig33_s))
        red = color[:,0]
        green = color[:,1]
        blue = color[:,2]
        print("axial stress:", np.sum(data.grainvolume*data.sig33_s)/np.sum(data.grainvolume), "MPa") 
    elif sym == "s11":
        try:
            colourbar(minimum,maximum,step,'MPa')
            norm = colors.Normalize(minimum,maximum)
        except:
            colourbar(-50,150,50,'MPa')
            norm = colors.Normalize(-50,150)
        color = cm.jet(norm(data.sig11_s))
        red = color[:,0]
        green = color[:,1]
        blue = color[:,2]
        print("transverse s11 stress:", np.sum(data.grainvolume*data.sig11_s)/np.sum(data.grainvolume), "MPa") 
    elif sym == "s22":
        try:
            colourbar(minimum,maximum,step,'MPa')
            norm = colors.Normalize(minimum,maximum)
        except:
            colourbar(-50,150,50,'MPa')
            norm = colors.Normalize(-50,150)
        color = cm.jet(norm(data.sig22_s))
        red = color[:,0]
        green = color[:,1]
        blue = color[:,2]
        print("transverse s22 stress:", np.sum(data.grainvolume*data.sig22_s)/np.sum(data.grainvolume), "MPa") 
    elif sym == "s12":
        try:
            colourbar(minimum,maximum,step,'MPa')
            norm = colors.Normalize(minimum,maximum)
        except:
            colourbar(-50,50,50,'MPa')
            norm = colors.Normalize(-50,50)
        color = cm.jet(norm(data.sig12_s))
        red = color[:,0]
        green = color[:,1]
        blue = color[:,2]
        print("shear s12 stress:", np.sum(data.grainvolume*data.sig12_s)/np.sum(data.grainvolume), "MPa") 
    elif sym == "s13":
        try:
            colourbar(minimum,maximum,step,'MPa')
            norm = colors.Normalize(minimum,maximum)
        except:
            colourbar(-50,50,50,'MPa')
            norm = colors.Normalize(-50,50)
        color = cm.jet(norm(data.sig13_s))
        red = color[:,0]
        green = color[:,1]
        blue = color[:,2]
        print("shear s13 stress:", np.sum(data.grainvolume*data.sig13_s)/np.sum(data.grainvolume), "MPa") 
    elif sym == "s23":
        try:
            colourbar(minimum,maximum,step,'MPa')
            norm = colors.Normalize(minimum,maximum)
        except:
            colourbar(-50,50,50,'MPa')
            norm = colors.Normalize(-50,50)
        color = cm.jet(norm(data.sig23_s))
        red = color[:,0]
        green = color[:,1]
        blue = color[:,2]
        print("shear s23 stress:", np.sum(data.grainvolume*data.sig23_s)/np.sum(data.grainvolume), "MPa") 
    elif sym == "latt_rot":
        norm = colors.Normalize(0,0.5)
        color = cm.jet(norm(data.latt_rot))
        red = color[:,0]
        green = color[:,1]
        blue = color[:,2]
        colourbar(0.0,0.5,0.1,'deg')
    elif sym == "tz":
        norm = colors.Normalize(-0.1,0.1)
        color = cm.jet(norm(data.tz))
        red = color[:,0]
        green = color[:,1]
        blue = color[:,2]
    elif sym == "vol":
        norm = colors.Normalize(0,10)
        color = cm.jet(norm(data.grainvolume/data.d_grainvolume))
        red = color[:,0]
        green = color[:,1]
        blue = color[:,2]
    elif sym == "tth":
        norm = colors.Normalize(0.007,0.009)
        color = cm.jet(norm(data.sig_tth/data.grainvolume**.2))
        red = color[:,0]
        green = color[:,1]
        blue = color[:,2]
        print(min(data.sig_tth/data.grainvolume**.2),max(data.sig_tth/data.grainvolume**.2))
#        pl.figure(8)
#        pl.plot(np.log(data.grainvolume),np.log(data.sig_tth),'.')
    elif sym == "eta":
        norm = colors.Normalize(0.08,0.15)
        color = cm.jet(norm(data.sig_eta/data.grainvolume**.2))
        red = color[:,0]
        green = color[:,1]
        blue = color[:,2]
        print(min(data.sig_eta/data.grainvolume**.2),max(data.sig_eta/data.grainvolume**.2))
#        pl.figure(8)
#        pl.plot(np.log(data.grainvolume),np.log(data.sig_eta),'.')
    else:
        np.random.seed(0)
        red = np.random.rand(data.nrows)
        np.random.seed(1)
        green = np.random.rand(data.nrows)
        np.random.seed(2)
        blue = np.random.rand(data.nrows)
    data.addcolumn(red,'red')
    data.addcolumn(green,'green')
    data.addcolumn(blue,'blue')
    return (data)
 
 
  def scale(data,scale):
    """
    scale grain sizes
    """
    if scale != None:
        data.addcolumn(data.size*scale,'size')
            
  
  def colourbar(min,max,step,legend):
    """
    add colourbar to plot with min, max and legend
    """
    fig2 = pl.figure(0)
    ax = fig2.add_subplot(111,visible=False)
    data = np.clip(np.random.randn(250,250),-1,1)
    cax = ax.imshow(data,interpolation='nearest',cmap=cm.jet)
    numticks = list(range(min,max+step,step))
    textticks = []
    num2ticks = []
    for i in numticks:
        textticks.append('%s' %i)
        num2ticks.append(2.*(i-(min+max)/2.)/(max-min))
#    print num2ticks
#    print textticks
    cbar = fig2.colorbar(cax,ticks=num2ticks,orientation='horizontal')
    cbar.ax.set_xticklabels(textticks)
    cbar.ax.set_title('[%s]' %legend,va='baseline')
  
  
  def plot3Dgrains(data,dim,view=None,label=None,data2=None,label2=None):
    """
    Plots xyz of grains in 3D
    Jette Oddershede March 4th 2009
    """
    if view == "z":
        elev = 90
        azim = 270
    elif view == "y":
        elev = 0
        azim = 270
    else:    
        elev = 0
        azim = 0
    fig = pl.figure(figsize=pl.figaspect(1.0))
    ax = p3.Axes3D(fig)
    #plot data
    color = []
    for i in range(data.nrows):
        color.append([data.red[i],data.green[i],data.blue[i]])
    cax=ax.scatter3D(data.x,data.y,data.z,s=data.size,c=color)
    if data2 != None:
        color2 = []
        for i in range(data2.nrows):
            color2.append([data2.red[i],data2.green[i],data2.blue[i]])
        ax.scatter3D(data2.x,data2.y,data2.z,s=data2.size,c=color)
    ax.set_xlabel('x')
    ax.set_ylabel('y')
    ax.set_zlabel('z')
    ax.set_xlim3d(-dim,dim)
    ax.set_ylim3d(-dim,dim)
    ax.set_zlim3d(-dim,dim)
    ax.view_init(elev, azim) 
    if label != None:
        for i in range(data.nrows):
            ax.text(data.x[i],data.y[i],data.z[i],'%i' %(data.grainno[i]))
    if data2 != None:
        if label2 != None:
           for i in range(data2.nrows):
                ax.text(data2.x[i],data2.y[i],data2.z[i],'%i' %(data2.grainno[i]))
    pl.title('%s    %s'%(options.input,options.sym))
    pl.show()
    
  options = get_options()
  (data) = read(options.input,options.sym,options.minimum,options.maximum,options.step)
  if options.input2 != None:
    (data2) = read(options.input2,options.sym2)  
    scale(data2,options.scale2)
  scale(data,options.scale)
  if options.text == "no" or options.text == "NO" or options.text == "No" or options.text == "n" or options.text == "N":
    options.text = None
  if options.input2 == None:
    plot3Dgrains(data,options.dim,options.view,label=options.text)
  else:
    plot3Dgrains(data,options.dim,options.view,label=options.text,data2=data2,label2=options.text2)

    

    


    

    



#!D:\bld\xfab_1644315860030\_h_env\python.exe

'''
grainspotter_loop

Runs GrainSpotter until no more good seed grains are found
Should enter one variable: the full path to the script's INI file
Like: /home/jules/sp8_2013/inis/script.ini
Must be run in folder where n00.flt is located
'''

from __future__ import absolute_import
from __future__ import print_function
import sys
import os
import shutil
import re
from string import split,digits,letters#,replace


GS='grainspotter'
GFF2UBI='gff_to_ubi.py'
RUNMAKEMAP='makemap_all.py'
MAKEGVE='make_gve.py'



# read and check input file
if len(sys.argv) == 1:
    print('Provide path to SCRIPT.ini')
elif len(sys.argv) != 2:
    print('Wrong number of arguments entered')
    sys.exit()
else:
    pathtoINI = sys.argv[1]
    print('\nCorrect number of arguments entered -- using %s \n' %pathtoINI)
    
f = open(pathtoINI,'r')
input = f.readlines()
f.close()

INIFILE=False
for line in input:
    if 'INIFILE' in line:
        INIFILE = eval(split(line,'=')[1])
        break
if INIFILE==False:
    print('Missing INIFILE variable - update ini file and restart')
else:
    print(' GrainSpotter ini file: %s' %INIFILE)
    f = open(INIFILE,'r')
    input = f.readlines()
    f.close()
    for line in input:
        if 'omegarange' in line:
            minome = eval(split(line)[1])
            maxome = eval(split(line)[2])
            break
    if minome < -180 or maxome > 180:
        print('\n OUT OF RANGE: omegarange (%s) must be within -180 to +180.' % INIFILE)
        sys.exit()
    print(' Omega range: %0.2f to %0.2f' %(minome,maxome))

f = open(pathtoINI,'r')
input = f.readlines()
f.close()
    
fltfile=False
for line in input:
    if 'fltfile' in line:
        fltfile = eval(split(line,'=')[1])
        break
if fltfile==False:
    print('Missing fltfile variable - update ini file and restart')
else:
    print(' GrainSpotter input peaks: %s' %fltfile)
    
ubifile=False
for line in input:
    if 'ubifile' in line:
        ubifile = eval(split(line,'=')[1])
        prefix  = split(ubifile,'.')[0].translate(None,digits)
        itemnum = eval(split(ubifile,'.')[0].translate(None,letters))
        break
if ubifile==False:
    print('Missing ubifile variable - update ini file and restart')
else:
    print(' GrainSpotter output grains: %s' %ubifile)
    
NUMFOUND=False
for line in input:
    if 'NUMFOUND' in line:
        NUMFOUND = eval(split(line,'=')[1])
        break
if NUMFOUND==False:
    print('Missing NUMFOUND variable - update ini file and restart')
else:
    print(' Continue running GrainSpotter while the number of new grains is at least: %s' %NUMFOUND)
    
PARS=False
for line in input:
    if 'PARS' in line:
        PARS = eval(split(line,'=')[1])
        break
if PARS==False:
    print('Missing PARS variable - update ini file and restart')
else:
    print(' Full path to par file is: %s' %PARS)
    
ICUTS=False
for line in input:
    if 'ICUTS' in line:
        ICUTS = eval(split(line,'=')[1])
        break
if ICUTS==False:
    print('Missing ICUTS variable - update ini file and restart')
else:
    print(' Makemap initial cuts: %s' %ICUTS)
    
FCUTS=False
for line in input:
    if 'FCUTS' in line:
        FCUTS = eval(split(line,'=')[1])
        break
if FCUTS==False:
    print('Missing FCUTS variable - update ini file and restart')
else:
    print(' Makemap final cuts: %s' %FCUTS)
    
SLIMIT=False
for line in input:
    if 'SLIMIT' in line:
        SLIMIT = eval(split(line,'=')[1])
        break
if SLIMIT==False:
    print('Missing SLIMIT variable - update ini file and restart')
else:
    print(' Makemap stop limit, number of new reflections per grain: %s' %SLIMIT)

MINPEAKS=False
for line in input:
    if 'MINPEAKS' in line:
        MINPEAKS = eval(split(line,'=')[1])
        break
if MINPEAKS==False:
    print('Missing MINPEAKS variable - update ini file and restart')
else:
    print(' Makemap minimum number of peaks: %s' %MINPEAKS)

CRYSYMM=False
for line in input:
    if 'CRYSYMM' in line:
        CRYSYMM = eval(split(line,'=')[1])
        break
if CRYSYMM==False:
    print('Missing CRYSYMM variable - update ini file and restart')
else:
    print(' The crystal symmetry is: %s' %CRYSYMM)    
    
EULERSTEP=False
for line in input:
    if 'EULERSTEP' in line:
        ES = split(line,'=')[1]
        EULERSTEP = eval(ES.replace('(','[').replace(')',']').replace(' ',','))
        break
if EULERSTEP==False:
    print('Missing EULERSTEP variable - update ini file and restart')

NSIGMAS=False
for line in input:
    if 'NSIGMAS' in line:
        NS = split(line,'=')[1]
        NSIGMAS = eval(NS.replace('(','[').replace(')',']').replace(' ',','))
        break
if NSIGMAS==False:
    print('Missing NSIGMAS variable - update ini file and restart')

    
# Setup for loop
testCon=0
testLim=3
loopCounter=0
numberofgrainsbefore=0
numberofgrains=-1
spottercounter=-1


while testCon < testLim:
    grains_in = '%s%0.2d.ubi' %(prefix,itemnum+loopCounter)
    grains_out = '%s%0.2d.ubi' %(prefix,itemnum+loopCounter)
    pks_in = '%s%0.2d.flt' %(prefix,itemnum-1+loopCounter)    
    pks_out = '%s%0.2d.flt' %(prefix,itemnum+loopCounter)
    log_out = '%s%0.2d.log' %(prefix,itemnum+loopCounter)
    gff_out = '%s%0.2d.gff' %(prefix,itemnum+loopCounter)
    
    print('\nConvert peaks to g-vectors')
    command = '%s %s rest.gve %s' %(MAKEGVE,pks_in,PARS)
    #print command
    os.system(command)

    if numberofgrains < NUMFOUND:
        spottercounter = spottercounter + 1        
    if spottercounter >= len(EULERSTEP) or spottercounter >= len(NSIGMAS):
        sys.exit(0)

    eulerstep = EULERSTEP[spottercounter]
    nsigmas = NSIGMAS[spottercounter]
    print('\nRunning GrainSpotter')
    print('spottercounter is: %i' %spottercounter)
    print('Now using eulerstep: %0.1f and nsigmas %0.1f' %(eulerstep,nsigmas))
    
    ## REPLACES cat COMMAND IN ORDER TO FUNCTION ON WINDOWS
    ## mkak 14.03.2016
    NEWINIFILE = 'grainspotter_current.ini'
    shutil.copyfile(INIFILE, NEWINIFILE)
    with open(NEWINIFILE, 'r') as sources:
        lines = sources.readlines()
    with open(NEWINIFILE, 'w') as sources:
        for line in lines:
            sources.write(re.sub(r'_EULERSTEP_', '%0.6f' % eulerstep, line))
    with open(NEWINIFILE, 'r') as sources:
        lines = sources.readlines()
    with open(NEWINIFILE, 'w') as sources:
        for line in lines:
            sources.write(re.sub(r'_NSIGMAS_', '%0.6f' % nsigmas, line))

    
    command = '%s grainspotter_current.ini > spotterOutTemp.txt' %GS
    os.system(command) 
    f = open('spotter_out.log','r')
    lines = f.readlines()
    f.close()
    print(lines[0], 'GrainSpotter done')
    numberofgrainsbefore = numberofgrains
    numberofgrains = eval(lines[0].translate(None,letters))
    slimit = SLIMIT*numberofgrains    
    
    
    ## REPLACES os.rename WITH shututil.move IN ORDER TO FUNCTION THE SAME ON ALL OS
    ## mkak 17.03.2016
    shutil.move("spotter_out.gff",gff_out)
    shutil.move("spotter_out.log",log_out)
    os.remove('spotter_out.ubi')

    if numberofgrains != 0:
        command = '%s %s %s' %(GFF2UBI,gff_out,grains_in)
        os.system(command)
        print('\nRunning makemap.py')
        command = '%s %s %s %s %f %f %i %i %s %s' %(RUNMAKEMAP,grains_in,pks_in,pks_out,ICUTS,FCUTS,slimit,MINPEAKS,PARS,CRYSYMM)
        os.system(command)
        
        if os.path.exists(grains_in):
            testCon = 0
            loopCounter = loopCounter + 1
        else:
            testCon = testCon + 1
    else:
        print('No grains found')
        os.remove(gff_out)
        os.remove(log_out)
        testCon = testCon + 1
        
os.remove('spotterOutTemp.txt')
print('\nOK\n')

        


import logging

def get_module_level_logger(module):
    """Build and set up formatting for a module level logger.

    This is meant to be called in each xfab module as:

        logger = get_module_level_logger(__name__)

    providing a module level logger to use for all local logging.

    The reason for module level loggers is to allow the end user to see what modules produced the logs
    and give controll over the level of logging. For instance,

        logging.getLogger('xfab.structure').setLevel(CRITICAL)

    will only display logging messages which are errors (or above in severity). Likewise following will shut down
    logging from xfab.structure completely:

        logging.getLogger('xfab.structure').disabled = True

    Some more logger docs can be found here: https://docs.python.org/3/howto/logging.html#configuring-logging

    Args:
        module (:obj:`str`): The module name and parents. e.g 'xfab.structure'.

    Returns:
        (:obj:`logging.logger`) A module level logger.

    """
    # create logger on lowest level=NOTSET
    logger = logging.getLogger(module)
    logger.setLevel(logging.NOTSET)

    # create console handler and set level
    console_handler = logging.StreamHandler()
    console_handler.setLevel(logging.NOTSET)

    # Some formatting printing what module is logging and the severity etc
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

    # add the formatter to the console handler
    console_handler.setFormatter(formatter)

    # add the console handler to logger
    logger.addHandler(console_handler)

    return logger
